class PolySol():
    def __init__(self):
        self.dim = 0
        self.ind = 0
        self.m = 0
        self.t_real = 0.0
        self.t_imag = 0.0
        self.sols = []
        self.err = 0.0
        self.rco = 0.0
        self.res = 0.0

    def read_sol(self, f, f_line):
        self.read_sol_ind(f_line)

        self.read_var_value(f.readline())
        
        self.read_sol_mul(f.readline())
        
        f.readline()

        dim = 0
        while True:
            f_line = f.readline()
            if f_line[0] != '=':
                self.read_var_value(f_line)
                dim += 1
            else:
                break;
        self.dim = dim

        self.info_var_value(f_line)

    def read_sol_ind(self, line):
        line = line[8:].split(':')
        self.ind = int(line[0])

    def read_sol_mul(self, line):
        line = line.split(':')
        self.m = int(line[1])
        
    def read_var_value(self,var_string):
        l = len(var_string)
        i = 0
        var_name = ''
        while True:
            c = var_string[i]
            i = i + 1
            if c == ':':
                break
            elif c != ' ':
                var_name += c


        new_element = ' '
        var_real = 0.0
        var_imag = 0.0
        var_end = 1

        while True:
            if var_string[i] != ' ':
                break
            i += 1

        real_start  = i

        while True:
            if var_string[i] == ' ':
                break;
            i += 1

        var_real = float(var_string[real_start:i])
        var_imag = float(var_string[i:])

        if var_name != 't':
            self.sols.append([var_name, var_real, var_imag])
        else:
            self.t_real = var_real
            self.t_imag = var_imag

        
    def info_var_value(self,info_string):
        j = 0
        i = 6
        start_i = 6
        var = [0.0,0.0,0.0]
        for j in range(3):
            var_start = 0
            while True:
                if var_start:
                    if info_string[i] == '=':
                        var[j] = float(info_string[start_i:i])
                        break;
                else:
                    if info_string[i] == ':':
                        var_start = 1
                        start_i = i+1
                i += 1

        self.err = var[0]
        self.rco = var[1]
        self.res = var[2]

    def print_sol(self):
        print 'dim = ', self.dim
        print 'ind = ', self.ind
        print 'm = ', self.m
        print 't_real = ', self.t_real
        print 't_m = ', self.t_imag
        print 'sol = ', self.sols
        print 'err = ', self.err
        print 'rco = ', self.rco
        print 'res = ', self.res

    def print_sol_table(self):
        print """<tr style ="border:1px solid black;"><td style="width:20%%"><b>SOL %d</b></td><td colspan="2", align = "right">  Mult: %d err: %.2e   rco:%.2e   res:%.2e</td></tr>"""%(self.ind,self.m, self.err,self.rco,self.res)
        for i in xrange(self.dim):
            print """<tr class="sol_high"><td>%s</td><td align="right">%.14e</td>\
                         <td align="right">%.14e</td></tr>"""\
                    %(self.sols[i][0],self.sols[i][1],self.sols[i][2])

            print """<tr class="sol_low"><td>%s</td><td align="right">%.5e</td>\
                         <td align="right">%.5e</td></tr>"""\
                    %(self.sols[i][0],self.sols[i][1],self.sols[i][2])


def read_sol_file(filename, sol_page):
    from phc_config import n_sol_page
    with open(filename) as f:
        while True:
            a = f.readline()
            if a[:5] == '=====':
                break
        print """<table style ="border-collapse: collapse;border: 1px solid black;">"""
        sol_start = sol_page*n_sol_page
        sol_end = (sol_page+1)*n_sol_page
        sols = []
        n_sol = 0
        while True:
            f_line = f.readline()
            if f_line =='':
                break
            s = PolySol()
            # this part need to be changed, don't read sols, skip sols before sol_start
            s.read_sol(f,f_line)
            if n_sol < sol_start:
                sols.append('')
            else:
                sols.append(s)
            n_sol += 1
            if(len(sols) > sol_end):
                break

        if (sol_start + n_sol_page > len(sols)):
            sol_end = len(sols)
        else:
            sol_end = sol_start + n_sol_page

        for i in range(sol_start, sol_end):
            sols[i].print_sol_table()

        print '</table>'

def read_sol_number_file(filename):
    with open(filename) as f:
        sol_symbol = '========='
        tmp_line = ''
        while True:
            a = f.readline()
            if ((len(a)>len(sol_symbol)  and a[:len(sol_symbol)] == sol_symbol)) or (a == ''):
                n_sol_line = last_line
                break
            last_line = a
        if n_sol_line != '':
            n_sol_line = n_sol_line.split(' ')
            return int(n_sol_line[0])
        return 0
    



"""
This script contains the timings in dictionary formats for runs
-- see the files run_mgs2_dd_new_k20c.txt and run_mgs2_qd_new_k20c.txt,
for the raw outputs of time and the command line arguments of run_mgs2_d ---
with run_mgs2_dd and run_mgs2_qd for running Newton's method on 
the chandra system for dimension 1024.
Running the script computes the speedups and
displays the results in a format that is easy to cut and paste
into a LaTeX table.
"""

def seconds(strtime):
   """
   Splits of the 'm' and the 's' in the time string
   to return the number of seconds as a float.
   """
   m = strtime.split('m')
   s = int(m[0])*60 + float(m[1][0:-1])
   return s

Cdd = {}
Cdd[1024] = {'real':'42m30.192s', 'user':'42m26.101s', 'sys':'0m0.039s'}
Gdd = {}
Gdd[(1024,64)] = {'real':'1m1.815s', 'user':'0m52.883s', 'sys':'0m8.676s'}
Cqd = {}
Cqd[1024] = {'real':'256m41.262s', 'user':'256m18.870s', 'sys':'0m0.074s'}
Gqd = {}
Gqd[(1024,64)] = {'real':'17m34.150s', 'user':'14m4.030s', 'sys':'3m28.215s'}

print (1024,'dd cpu'), Cdd[1024]['real'], Cdd[1024]['user'], Cdd[1024]['sys']
Sdd = list(Gdd.keys())
Sdd.sort()
for L in Sdd:
   spdup = seconds(Cdd[L[0]]['real'])/seconds(Gdd[L]['real'])
   print L, Gdd[L]['real'], Gdd[L]['user'], Gdd[L]['sys'], ':' , \
         seconds(Cdd[L[0]]['real']) , '/' , seconds(Gdd[L]['real']), \
         '=', spdup
print (1024,'qd cpu'), Cqd[1024]['real'], Cqd[1024]['user'], Cqd[1024]['sys']
Sqd = list(Gqd.keys())
Sqd.sort()
for L in Sqd:
   spdup = seconds(Cqd[L[0]]['real'])/seconds(Gqd[L]['real'])
   print L, Gqd[L]['real'], Gqd[L]['user'], Gqd[L]['sys'], ':' , \
         seconds(Cqd[L[0]]['real']) , '/' , seconds(Gqd[L]['real']), \
         '=', spdup



n = range(16,257,16)
c = [2.26,16.48,53.03,123.80,238.83,409.30,649.30,962.17,1363.57,1867.36,\
     2480.18, 3221.12, 4080.77, 5099.98, 6253.65, 7575.85]
g = [3.36, 5.58,10.26, 15.16, 22.45, 28.64,36.59,45.29,59.48,70.54,\
     84.26, 97.62, 112.19, 128.11, 146.31, 164.33]
for k in range(0,len(c)):
   s = '%4d' % n[k]
   s = s + ' ' + ('%5.2f' % c[k]).rjust(8)
   s = s + ' ' + ('%5.2f' % g[k]).rjust(7)
   s = s + ' ' + ('%5.2f' % (c[k]/g[k])).rjust(6)
   print s

# Below is the screen output of the Python script run_mgs_timings.py
# comparing the running times of the modified Gram-Schmidt method 
# for the NVIDIA Tesla K20C GPU with a 2.60GHz CPU,
# done with complex double precision arithmetic.

# $ python run_mgs_timings.py
# running the modified Gram-Schmidt method
# type precision level (d, dd, or qd) : d
# give start dimension (>= 2) : 16
# give stop dimension (<= 256) : 256
# give the step : 16
# give frequency per dimension (>= 1) : 10000
# give the name of the output file : run_mgs_d_times.txt
# running for dimension 16 and frequency 10000 on CPU
# ['real 2.25', 'user 2.23', 'sys 0.02']
# running for dimension 16 and frequency 10000 on GPU
# ['real 1.23', 'user 0.45', 'sys 0.63']
# running for dimension 32 and frequency 10000 on CPU
# ['real 16.31', 'user 16.20', 'sys 0.08']
# running for dimension 32 and frequency 10000 on GPU
# ['real 6.51', 'user 3.88', 'sys 2.50']
# running for dimension 48 and frequency 10000 on CPU
# ['real 53.12', 'user 52.88', 'sys 0.15']
# running for dimension 48 and frequency 10000 on GPU
# ['real 10.24', 'user 6.40', 'sys 3.67']
# running for dimension 64 and frequency 10000 on CPU
# ['real 123.70', 'user 123.19', 'sys 0.31']
# running for dimension 64 and frequency 10000 on GPU
# ['real 2.55', 'user 1.66', 'sys 0.71']
# running for dimension 80 and frequency 10000 on CPU
# ['real 239.62', 'user 238.34', 'sys 0.88']
# running for dimension 80 and frequency 10000 on GPU
# ['real 22.41', 'user 13.94', 'sys 8.20']
# running for dimension 96 and frequency 10000 on CPU
# ['real 409.30', 'user 407.17', 'sys 1.47']
# running for dimension 96 and frequency 10000 on GPU
# ['real 28.64', 'user 17.94', 'sys 10.50']
# running for dimension 112 and frequency 10000 on CPU
# ['real 649.00', 'user 645.68', 'sys 2.29']
# running for dimension 112 and frequency 10000 on GPU
# ['real 36.59', 'user 22.38', 'sys 13.87']
# running for dimension 128 and frequency 10000 on CPU
# ['real 962.17', 'user 957.79', 'sys 2.71']
# running for dimension 128 and frequency 10000 on GPU
# ['real 45.29', 'user 28.02', 'sys 16.99']
# running for dimension 144 and frequency 10000 on CPU
# ['real 1363.57', 'user 1358.10', 'sys 3.30']
# running for dimension 144 and frequency 10000 on GPU
# ['real 59.48', 'user 36.73', 'sys 22.45']
# running for dimension 160 and frequency 10000 on CPU
# ['real 1867.36', 'user 1858.74', 'sys 5.56']
# running for dimension 160 and frequency 10000 on GPU
# ['real 70.54', 'user 42.69', 'sys 27.56']
# running for dimension 176 and frequency 10000 on CPU
# ['real 2480.18', 'user 2469.74', 'sys 6.24']
# running for dimension 176 and frequency 10000 on GPU
# ['real 84.26', 'user 51.40', 'sys 32.52']
# running for dimension 192 and frequency 10000 on CPU
# ['real 3221.12', 'user 3206.63', 'sys 8.81']
# running for dimension 192 and frequency 10000 on GPU
# ['real 97.62', 'user 58.97', 'sys 38.17']
# running for dimension 208 and frequency 10000 on CPU
# ['real 4080.77', 'user 4064.61', 'sys 9.46']
# running for dimension 208 and frequency 10000 on GPU
# ['real 112.19', 'user 67.24', 'sys 44.56']
# running for dimension 224 and frequency 10000 on CPU
# ['real 5099.98', 'user 5077.51', 'sys 13.65']
# running for dimension 224 and frequency 10000 on GPU
# ['real 128.11', 'user 76.88', 'sys 50.85']
# running for dimension 240 and frequency 10000 on CPU
# ['real 6253.65', 'user 6230.79', 'sys 12.75']
# running for dimension 240 and frequency 10000 on GPU
# ['real 146.31', 'user 88.93', 'sys 56.93']
# running for dimension 256 and frequency 10000 on CPU
# ['real 7575.85', 'user 7549.78', 'sys 14.52']
# running for dimension 256 and frequency 10000 on GPU
# ['real 164.33', 'user 99.73', 'sys 64.22']
# $ 



"""
The perfidious polynomial of Wilkinson of degree d is the product of the
first d factor x - k, for k ranging from 1 to d.
"""
def wilkpol(deg):
    """
    Returns the string representation of the Wilkinson polynomial
    of degree deg > 0.
    """
    result = "(x - 1)"
    for k in range(2, deg+1):
        factor = '*(x - %d)' % k
        result = result + factor
    result = result + ';'
    return result

def solve(pol):
    """
    Applies the blackbox solver of phcpy to the polynomial pol.
    """
    from phcpy.solver import solve
    try:
        sols = solve([pol], verbose=False)
        print sols
        return sols
    except:
        print 'an exception happened in the blackbox solver'
        return []

def track(pol):
    """
    Applies the series-Pade tracker to solve the polynomial,
    either in a step-by-step manner or without this interaction.
    """
    from phcpy.solver import total_degree_start_system as tdss
    from phcpy.curves import tune_homotopy_continuation_parameters
    from phcpy.curves import standard_next_track, standard_track
    (startpol, startsols) = tdss([pol])
    print 'the start polynomial :'
    print startpol
    print 'the start solutions :'
    for sol in startsols:
        print sol
    tune_homotopy_continuation_parameters()
    ans = raw_input("Interactive, step-by-step track ? (y/n) ");
    if(ans == 'y'):
        sols = standard_next_track([pol], startpol, startsols, True)
    else:
        sols = standard_track([pol], startpol, startsols, \
                              filename="/tmp/outoftrack", verbose=True)
    print 'the computed solutions :'
    for sol in sols:
        print sol

def main():
    """
    Prompts the user for a degree and then computes the roots
    of the Wilkinson polynomial.
    """
    deg = int(raw_input("Give the degree : "))
    pol = wilkpol(deg)
    print pol
    ans = raw_input("Apply the blackbox solver ? (y/n) ")
    if(ans == 'y'):
        sols = solve(pol)
        for sol in sols:
            print sol
    track(pol)

main()



"""
Exports functions to compute mixed volumes and stable mixed volumes.
The mixed volume of a polynomial system is a generically sharp upper bound
on the number of isolated solutions with nonzero coordinates.
Stable mixed volumes count solutions with zero coordinates as well.
Polyhedral homotopies solve random coefficient systems,
tracking exactly as many paths as the mixed volume.
"""
from ctypes import c_int32, c_double, pointer
from phcpy.version import get_phcfun
from phcpy.polynomials import set_double_system, get_double_system
from phcpy.polynomials import set_double_double_system
from phcpy.polynomials import get_double_double_system
from phcpy.polynomials import set_quad_double_system
from phcpy.polynomials import get_quad_double_system
from phcpy.solutions import get_double_solutions, verify
from phcpy.solutions import clear_double_solutions
from phcpy.solutions import get_double_double_solutions
from phcpy.solutions import clear_double_double_solutions
from phcpy.solutions import get_quad_double_solutions
from phcpy.solutions import clear_quad_double_solutions

def compute_mixed_volume(demics=True, vrblvl=0):
    """
    Returns the mixed volume of the polynomial system,
    set in double precision.
    The demics flag indicates if dynamic enumeration as implemented by
    the software DEMiCs will be used, otherwise MixedVol is called.
    The verbose level is given by vrblvl.
    """
    if vrblvl > 0:
        print('in compute_mixed_volume, demics flag :', demics)
    phc = get_phcfun(vrblvl-1)
    mixvol = pointer(c_int32(0))
    bbb = pointer(c_int32(0))
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> compute_mixed_volume calls phc', end='')
    if demics:
        retval = phc(843, mixvol, bbb, ccc, vrb)
    else:
        retval = phc(78, mixvol, bbb, ccc, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
    return mixvol[0]

def compute_stable_mixed_volume(demics=True, vrblvl=0):
    """
    Returns the mixed and the stable mixed volume of the polynomial system
    set in double precision.
    The demics flag indicates if dynamic enumeration as implemented by
    the software DEMiCs will be used, otherwise MixedVol is called.
    The verbose level is given by vrblvl.
    """
    if vrblvl > 0:
        print('in compute_stable_mixed_volume, demics flag :', demics)
    phc = get_phcfun(vrblvl-1)
    mixvol = pointer(c_int32(0))
    stablemv = pointer(c_int32(0))
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> compute_stable_mixed_volume calls phc', end='')
    if demics:
        retval = phc(844, mixvol, stablemv, ccc, vrb)
    else:
        retval = phc(79, mixvol, stablemv, ccc, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
    return (mixvol[0], stablemv[0])

def mixed_volume(pols, demics=True, vrblvl=0):
    """
    Returns the mixed volume of the polynomial system in the list pols.
    The polynomial system must have as many equations as unknowns.
    The demics flag indicates if dynamic enumeration as implemented by
    the software DEMiCs will be used, otherwise MixedVol is called.
    The verbose level is given by vrblvl.
    """
    if vrblvl > 0:
        print('in mixed_volume, demics flag :', demics)
        print('the polynomials :')
        for pol in pols:
            print(pol)
    set_double_system(len(pols), pols, vrblvl-1)
    return compute_mixed_volume(demics, vrblvl)

def stable_mixed_volume(pols, demics=True, vrblvl=0):
    """
    Returns the mixed and the stable mixed volume of the polynomial system
    in the list pols.
    The polynomial system must have as many equations as unknowns.
    The demics flag indicates if dynamic enumeration as implemented by
    the software DEMiCs will be used, otherwise MixedVol is called.
    The verbose level is given by vrblvl.
    """
    if vrblvl > 0:
        print('in stable_mixed_volume, demics flag :', demics)
        print('the polynomials :')
        for pol in pols:
            print(pol)
    set_double_system(len(pols), pols, vrblvl-1)
    return compute_stable_mixed_volume(demics, vrblvl)

def clear_cells(vrblvl=0):
    """
    Clears the computed mixed cells.
    """
    if vrblvl > 0:
        print('in clear_cells ...')
    phc = get_phcfun(vrblvl-1)
    aaa = pointer(c_int32(0))
    bbb = pointer(c_int32(0))
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> clear_cells calls phc', end='')
    retval = phc(94, aaa, bbb, ccc, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
    return retval

def are_cells_stable(vrblvl=0):
    """
    Returns True if stable mixed cells were computed,
    returns False otherwise.
    """
    if vrblvl > 0:
        print('in are_cells_stable ...')
    phc = get_phcfun(vrblvl-1)
    aflag = pointer(c_int32(0))
    bbb = pointer(c_int32(0))
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> are_cells_stable calls phc', end='')
    retval = phc(879, aflag, bbb, ccc, vrb)
    result = aflag[0] == 1
    if vrblvl > 0:
        print(', return value :', retval)
        print('the flag :', result)
    return result

def number_of_cells(vrblvl=0):
    """
    Returns the number of cells computed by the mixed_volume function.
    """
    if vrblvl > 0:
        print('in number_of_cells ...')
    phc = get_phcfun(vrblvl-1)
    alen = pointer(c_int32(0))
    bbb = pointer(c_int32(0))
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> number_of_cells calls phc', end='')
    retval = phc(82, alen, bbb, ccc, vrb)
    result = alen[0]
    if vrblvl > 0:
        print(', return value :', retval)
        print('the number of cells :', result)
    return result

def number_of_original_cells(vrblvl=0):
    """
    Returns the number of original cells,
    the cells without artificial origin,
    computed by the mixed_volume function.
    """
    if vrblvl > 0:
        print('in number_of_mixed_original_cells ...')
    phc = get_phcfun(vrblvl-1)
    alen = pointer(c_int32(0))
    bbb = pointer(c_int32(0))
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> number_of_original_cells calls phc', end='')
    retval = phc(880, alen, bbb, ccc, vrb)
    result = alen[0]
    if vrblvl > 0:
        print(', return value :', retval)
        print('the number of original cells :', result)
    return result

def number_of_stable_cells(vrblvl=0):
    """
    Returns the number of stable cells computed 
    by the mixed_volume function.
    """
    if vrblvl > 0:
        print('in number_of_mixed_cells ...')
    phc = get_phcfun(vrblvl-1)
    alen = pointer(c_int32(0))
    bbb = pointer(c_int32(0))
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> number_of_cells calls phc', end='')
    retval = phc(881, alen, bbb, ccc, vrb)
    result = alen[0]
    if vrblvl > 0:
        print(', return value :', retval)
        print('the number of cells :', result)
    return result

def cell_mixed_volume(idx, vrblvl=0):
    """
    Returns the mixed volume of cell with index idx.
    """
    if vrblvl > 0:
        print('in cell_mixed_volume, idx :', idx)
    phc = get_phcfun(vrblvl-1)
    aidx = pointer(c_int32(idx))
    bmv = pointer(c_int32(0))
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> cell_mixed_volume calls phc', end='')
    retval = phc(90, aidx, bmv, ccc, vrb)
    result = bmv[0]
    if vrblvl > 0:
        print(', return value :', retval)
        print('the mixed volume of cell', idx, 'is', result)
    return result

def double_random_coefficient_system(vrblvl=0):
    """
    Makes a random coefficient system using the type of mixture
    and the supports used to compute the mixed volume.
    Sets the system in double precision.
    """
    if vrblvl > 0:
        print('in double_random_coefficient_system ...')
    phc = get_phcfun(vrblvl-1)
    aaa = pointer(c_int32(0))
    bbb = pointer(c_int32(0))
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> double_random_coefficient_system calls phc', end='')
    retval = phc(96, aaa, bbb, ccc, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
    return retval

def double_double_random_coefficient_system(vrblvl=0):
    """
    Makes a random coefficient system using the type of mixture
    and the supports used to compute the mixed volume.
    Sets the system in double double precision.
    """
    if vrblvl > 0:
        print('in double_double_random_coefficient_system ...')
    phc = get_phcfun(vrblvl-1)
    aaa = pointer(c_int32(0))
    bbb = pointer(c_int32(0))
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> double_double_random_coefficient_system calls phc', end='')
    retval = phc(460, aaa, bbb, ccc, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
    return retval

def quad_double_random_coefficient_system(vrblvl=0):
    """
    Makes a random coefficient system using the type of mixture
    and the supports used to compute the mixed volume.
    Sets the system in quad double precision.
    """
    if vrblvl > 0:
        print('in quad_double_random_coefficient_system ...')
    phc = get_phcfun(vrblvl-1)
    aaa = pointer(c_int32(0))
    bbb = pointer(c_int32(0))
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> quad_double_random_coefficient_system calls phc', end='')
    retval = phc(470, aaa, bbb, ccc, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
    return retval

def set_double_coefficient_system(vrblvl=0):
    """
    Sets the constructed random coefficient system to the system
    in double precision.
    """
    if vrblvl > 0:
        print('in set_double_coefficient_system ...')
    phc = get_phcfun(vrblvl-1)
    aaa = pointer(c_int32(0))
    bbb = pointer(c_int32(0))
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> set_double_coefficient_system calls phc', end='')
    retval = phc(99, aaa, bbb, ccc, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
    return retval

def set_double_double_coefficient_system(vrblvl=0):
    """
    Sets the constructed random coefficient system to the system
    in double double precision.
    """
    if vrblvl > 0:
        print('in set_double_double_coefficient_system ...')
    phc = get_phcfun(vrblvl-1)
    aaa = pointer(c_int32(0))
    bbb = pointer(c_int32(0))
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> set_double_double_coefficient_system calls phc', end='')
    retval = phc(463, aaa, bbb, ccc, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
    return retval

def set_quad_double_coefficient_system(vrblvl=0):
    """
    Sets the constructed random coefficient system to the system
    in quad double precision.
    """
    if vrblvl > 0:
        print('in set_quad_double_coefficient_system ...')
    phc = get_phcfun(vrblvl-1)
    aaa = pointer(c_int32(0))
    bbb = pointer(c_int32(0))
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> set_quad_double_coefficient_system calls phc', end='')
    retval = phc(473, aaa, bbb, ccc, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
    return retval

def set_double_polyhedral_homotopy(vrblvl=0):
    """
    Sets the polyhedral homotopy in double precision based
    on the lifted supports used to compute the mixed volume.
    Initializes the data to store the solutions.
    """
    if vrblvl > 0:
        print('in set_double_polyhedral_homotopy ...')
    phc = get_phcfun(vrblvl-1)
    aaa = pointer(c_int32(0))
    bbb = pointer(c_int32(0))
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> set_double_polyhedral_homotopy calls phc', end='')
    retval = phc(101, aaa, bbb, ccc, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
    return retval

def set_double_double_polyhedral_homotopy(vrblvl=0):
    """
    Sets the polyhedral homotopy in double double precision based
    on the lifted supports used to compute the mixed volume.
    Initializes the data to store the solutions.
    """
    if vrblvl > 0:
        print('in set_double_double_polyhedral_homotopy ...')
    phc = get_phcfun(vrblvl-1)
    aaa = pointer(c_int32(0))
    bbb = pointer(c_int32(0))
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> set_double_double_polyhedral_homotopy calls phc', end='')
    retval = phc(465, aaa, bbb, ccc, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
    return retval

def set_quad_double_polyhedral_homotopy(vrblvl=0):
    """
    Sets the polyhedral homotopy in quad double precision based
    on the lifted supports used to compute the mixed volume.
    Initializes the data to store the solutions.
    """
    if vrblvl > 0:
        print('in set_quad_double_polyhedral_homotopy ...')
    phc = get_phcfun(vrblvl-1)
    aaa = pointer(c_int32(0))
    bbb = pointer(c_int32(0))
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> set_quad_double_polyhedral_homotopy calls phc', end='')
    retval = phc(475, aaa, bbb, ccc, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
    return retval

def double_solve_start_system(idx, vrblvl=0):
    """
    Solves the start system corresponding to the cell with index idx,
    using double precision arithmetic.
    Returns the number of solutions found,
    which must equal the mixed volume of the cell.
    """
    if vrblvl > 0:
        print('in double_solve_start_system, idx :', idx)
    phc = get_phcfun(vrblvl-1)
    aidx = pointer(c_int32(idx))
    bnbr = pointer(c_int32(0))
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> double_solve_start_system calls phc', end='')
    retval = phc(102, aidx, bnbr, ccc, vrb)
    result = bnbr[0]
    if vrblvl > 0:
        print(', return value :', retval)
        print('number of solutions found :', result)
    return result

def double_double_solve_start_system(idx, vrblvl=0):
    """
    Solves the start system corresponding to the cell with index idx,
    using double double precision arithmetic.
    Returns the number of solutions found,
    which must equal the mixed volume of the cell.
    """
    if vrblvl > 0:
        print('in double_double_solve_start_system, idx :', idx)
    phc = get_phcfun(vrblvl-1)
    aidx = pointer(c_int32(idx))
    bnbr = pointer(c_int32(0))
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> double_double_solve_start_system calls phc', end='')
    retval = phc(466, aidx, bnbr, ccc, vrb)
    result = bnbr[0]
    if vrblvl > 0:
        print(', return value :', retval)
        print('number of solutions found :', result)
    return result

def quad_double_solve_start_system(idx, vrblvl=0):
    """
    Solves the start system corresponding to the cell with index idx,
    using quad double precision arithmetic.
    Returns the number of solutions found,
    which must equal the mixed volume of the cell.
    """
    if vrblvl > 0:
        print('in quad_double_solve_start_system, idx :', idx)
    phc = get_phcfun(vrblvl-1)
    aidx = pointer(c_int32(idx))
    bnbr = pointer(c_int32(0))
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> quad_double_solve_start_system calls phc', end='')
    retval = phc(476, aidx, bnbr, ccc, vrb)
    result = bnbr[0]
    if vrblvl > 0:
        print(', return value :', retval)
        print('number of solutions found :', result)
    return result

def double_track_path(cellidx, pathidx, vrblvl=0):
    """
    Tracks path with index pathidx defined by cell with index cellidx.
    Prior to calling this function, the start system corresponding
    to the cell must have been solved, in double precision.
    """
    if vrblvl > 0:
        print('in double_track_path, cell :', cellidx, end='')
        print(', path :', pathidx)
    phc = get_phcfun(vrblvl-1)
    aidx = pointer(c_int32(cellidx))
    nbrs = (c_int32 * 2)()
    nbrs[0] = pathidx
    nbrs[1] = vrblvl
    bnbrs = pointer(nbrs)
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> double_track_path calls phc', end='')
    retval = phc(103, aidx, bnbrs, ccc, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
    return retval

def double_double_track_path(cellidx, pathidx, vrblvl=0):
    """
    Tracks path with index pathidx defined by cell with index cellidx.
    Prior to calling this function, the start system corresponding
    to the cell must have been solved, in double double precision.
    """
    if vrblvl > 0:
        print('in double_double_track_path, cell :', cellidx, end='')
        print(', path :', pathidx)
    phc = get_phcfun(vrblvl-1)
    aidx = pointer(c_int32(cellidx))
    nbrs = (c_int32 * 2)()
    nbrs[0] = pathidx
    nbrs[1] = vrblvl
    bnbrs = pointer(nbrs)
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> double_double_track_path calls phc', end='')
    retval = phc(467, aidx, bnbrs, ccc, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
    return retval

def quad_double_track_path(cellidx, pathidx, vrblvl=0):
    """
    Tracks path with index pathidx defined by cell with index cellidx.
    Prior to calling this function, the start system corresponding
    to the cell must have been solved, in quad double precision.
    """
    if vrblvl > 0:
        print('in quad_double_track_path, cell :', cellidx, end='')
        print(', path :', pathidx)
    phc = get_phcfun(vrblvl-1)
    aidx = pointer(c_int32(cellidx))
    nbrs = (c_int32 * 2)()
    nbrs[0] = pathidx
    nbrs[1] = vrblvl
    bnbrs = pointer(nbrs)
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> quad_double_track_path calls phc', end='')
    retval = phc(477, aidx, bnbrs, ccc, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
    return retval

def set_double_start_solution(cellidx, solidx, vrblvl=0):
    """
    Sets the start solution of index solidx, with cell with index cellidx,
    to the solutions in double precision.
    """ 
    if vrblvl > 0:
        print('in set_double_start_solution, cell :', cellidx, end='')
        print(', solidx :', solidx)
    phc = get_phcfun(vrblvl-1)
    aidx = pointer(c_int32(cellidx))
    bidx = pointer(c_int32(solidx))
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> set_double_start_solution calls phc', end='')
    retval = phc(104, aidx, bidx, ccc, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
    return retval

def set_double_double_start_solution(cellidx, solidx, vrblvl=0):
    """
    Sets the start solution of index solidx, with cell with index cellidx,
    to the solutions in double double precision.
    """ 
    if vrblvl > 0:
        print('in set_double_double_start_solution, cell :', cellidx, end='')
        print(', solidx :', solidx)
    phc = get_phcfun(vrblvl-1)
    aidx = pointer(c_int32(cellidx))
    bidx = pointer(c_int32(solidx))
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> set_double_double_start_solution calls phc', end='')
    retval = phc(468, aidx, bidx, ccc, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
    return retval

def set_quad_double_start_solution(cellidx, solidx, vrblvl=0):
    """
    Sets the start solution of index solidx, with cell with index cellidx,
    to the solutions in quad double precision.
    """ 
    if vrblvl > 0:
        print('in set_quad_double_start_solution, cell :', cellidx, end='')
        print(', solidx :', solidx)
    phc = get_phcfun(vrblvl-1)
    aidx = pointer(c_int32(cellidx))
    bidx = pointer(c_int32(solidx))
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> set_quad_double_start_solution calls phc', end='')
    retval = phc(478, aidx, bidx, ccc, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
    return retval

def double_polyhedral_homotopies(vrblvl=0):
    """
    Runs the polyhedral homotopies and returns a random coefficient
    system, in double precision arithmetic,
    based on the mixed volume computation
    """
    if vrblvl > 0:
        print('in double_polyhedral_homotopies ...')
    double_random_coefficient_system(vrblvl)
    set_double_coefficient_system(vrblvl)
    rndcffsys = get_double_system(vrblvl-1)
    set_double_polyhedral_homotopy(vrblvl)
    is_stable = are_cells_stable(vrblvl)
    if is_stable:
        nbcells = number_of_original_cells(vrblvl)
    else:
        nbcells = number_of_cells(vrblvl)
    if vrblvl > 0:
         print('the number of cells :', nbcells)
    clear_double_solutions(vrblvl-1)
    for cell in range(1, nbcells+1):
        nbsols = double_solve_start_system(cell, vrblvl)
        if vrblvl > 0:
            print(f'system {cell} has {nbsols} solutions')
        for idxsol in range(1, nbsols+1):
            if vrblvl > 0:
                print(f'-> tracking path {idxsol} out of {nbsols}')
            double_track_path(cell, idxsol, vrblvl)
            set_double_start_solution(cell, idxsol, vrblvl)
    sols = get_double_solutions(vrblvl-1)
    return (rndcffsys, sols)

def double_double_polyhedral_homotopies(vrblvl=0):
    """
    Runs the polyhedral homotopies and returns a random coefficient
    system, in double double precision arithmetic,
    based on the mixed volume computation
    """
    if vrblvl > 0:
        print('in double_double_polyhedral_homotopies ...')
    double_double_random_coefficient_system(vrblvl)
    set_double_double_coefficient_system(vrblvl)
    rndcffsys = get_double_double_system(vrblvl-1)
    set_double_double_polyhedral_homotopy(vrblvl)
    is_stable = are_cells_stable(vrblvl)
    if is_stable:
        nbcells = number_of_original_cells(vrblvl)
    else:
        nbcells = number_of_cells(vrblvl)
    if vrblvl > 0:
         print('the number of cells :', nbcells)
    clear_double_double_solutions(vrblvl-1)
    for cell in range(1, nbcells+1):
        nbsols = double_double_solve_start_system(cell, vrblvl)
        if vrblvl > 0:
            print(f'system {cell} has {nbsols} solutions')
        for idxsol in range(1, nbsols+1):
            if vrblvl > 0:
                print(f'-> tracking path {idxsol} out of {nbsols}')
            double_double_track_path(cell, idxsol, vrblvl)
            set_double_double_start_solution(cell, idxsol, vrblvl)
    sols = get_double_double_solutions(vrblvl-1)
    return (rndcffsys, sols)

def quad_double_polyhedral_homotopies(vrblvl=0):
    """
    Runs the polyhedral homotopies and returns a random coefficient
    system, in quad double precision arithmetic,
    based on the mixed volume computation
    """
    if vrblvl > 0:
        print('in quad_double_polyhedral_homotopies ...')
    quad_double_random_coefficient_system(vrblvl)
    set_quad_double_coefficient_system(vrblvl)
    rndcffsys = get_quad_double_system(vrblvl-1)
    set_quad_double_polyhedral_homotopy(vrblvl)
    is_stable = are_cells_stable(vrblvl)
    if is_stable:
        nbcells = number_of_original_cells(vrblvl)
    else:
        nbcells = number_of_cells(vrblvl)
    if vrblvl > 0:
         print('the number of cells :', nbcells)
    clear_quad_double_solutions(vrblvl-1)
    for cell in range(1, nbcells+1):
        nbsols = quad_double_solve_start_system(cell, vrblvl)
        if vrblvl > 0:
            print(f'system {cell} has {nbsols} solutions')
        for idxsol in range(1, nbsols+1):
            if vrblvl > 0:
                print(f'-> tracking path {idxsol} out of {nbsols}')
            quad_double_track_path(cell, idxsol, vrblvl)
            set_quad_double_start_solution(cell, idxsol, vrblvl)
    sols = get_quad_double_solutions(vrblvl-1)
    return (rndcffsys, sols)

def make_random_coefficient_system(pols, demics=True, precision='d', \
    vrblvl=0):
    """
    For a polynomial system in the list pols with as many equations
    as unknowns, computes the mixed volume (by default by DEMiCs)
    and runs the polyhedral homotopies to solve a random coefficient
    system in double, double double, or quad double precision,
    if the precision flag is set to 'd', 'dd', or 'qd' respectively.
    Returns a tuple with the mixed volume, random coefficient system,
    and the solutions of the random coefficient system.
    """
    if vrblvl > 0:
        print('make_random_coefficient_system, demics flag :', demics)
        print('precision :', precision)
        print('the polynomials :')
        for pol in pols:
            print(pol)
    mv = mixed_volume(pols, demics, vrblvl)
    if precision == 'd':
        rndcffsys, sols = double_polyhedral_homotopies(vrblvl)
    elif precision == 'dd':
        rndcffsys, sols = double_double_polyhedral_homotopies(vrblvl)
    elif precision == 'qd':
        rndcffsys, sols = quad_double_polyhedral_homotopies(vrblvl)
    else:
        print('Wrong value of precision flag.')
    return (mv, rndcffsys, sols)

def test_mixed_volume(vrblvl=0):
    """
    Computes the mixed volume of a simple example.
    The verbose level is given by vrblvl.
    """
    if vrblvl > 0:
        print('in test_mixed_volume ...')
    polynomials = ["x^3 + 2*x*y - 1;", "x + y - 1;"]
    mvl = mixed_volume(polynomials, True, vrblvl)
    nbr = number_of_cells(vrblvl)
    if vrblvl > 0:
        print('the mixed volume by DEMiCs :', mvl)
        print('the number of cells :', nbr)
    fail = int(mvl != 3)
    mvcells = 0
    for idx in range(1, nbr+1):
        mvcells = mvcells + cell_mixed_volume(idx, vrblvl)
    if vrblvl > 0:
        print('sum of mixed volumes of cells :', mvcells)
    fail = fail + int(mvcells != 3)
    clear_cells(vrblvl)
    mvl = mixed_volume(polynomials, False, vrblvl)
    nbr = number_of_cells(vrblvl)
    if vrblvl > 0:
        print('the mixed volume by MixedVol :', mvl)
        print('the number of cells :', nbr)
    fail = fail + int(mvl != 3)
    mvcells = 0
    for idx in range(1, nbr+1):
        mvcells = mvcells + cell_mixed_volume(idx, vrblvl)
    if vrblvl > 0:
        print('sum of mixed volumes of cells :', mvcells)
    fail = fail + int(mvcells != 3)
    return fail

def test_stable_mixed_volume(vrblvl=0):
    """
    Computes the stable mixed volume of a simple example.
    The verbose level is given by vrblvl.
    """
    if vrblvl > 0:
        print('in test_stable_mixed_volume ...')
    polynomials = ["x^3 + 2*x*y - x^2*y;", "x + y - x^3;"]
    mvl, smv = stable_mixed_volume(polynomials, True, vrblvl)
    nbr = number_of_cells(vrblvl)
    stbnbr = number_of_stable_cells(vrblvl)
    if vrblvl > 0:
        print('the mixed volume by DEMiCs :', mvl)
        print('the stable mixed volume by DEMiCs :', smv)
        print('the number of cells :', nbr)
        print('the number of stable cells :', stbnbr)
    mvcells = 0
    for idx in range(1, nbr+1):
        mvcells = mvcells + cell_mixed_volume(idx, vrblvl)
    if vrblvl > 0:
        print('sum of mixed volumes of cells :', mvcells)
    fail = int(mvl != 3) + int(smv != 5) + int(mvcells != 5)
    clear_cells(vrblvl)
    mvl, smv = stable_mixed_volume(polynomials, False, vrblvl)
    nbr = number_of_cells(vrblvl)
    stbnbr = number_of_stable_cells(vrblvl)
    if vrblvl > 0:
        print('the mixed volume by MixedVol :', mvl)
        print('the stable mixed volume by MixedVol :', smv)
        print('the number of cells :', nbr)
        print('the number of stable cells :', stbnbr)
    mvcells = 0
    for idx in range(1, nbr+1):
        mvcells = mvcells + cell_mixed_volume(idx, vrblvl)
    if vrblvl > 0:
        print('sum of mixed volumes of cells :', mvcells)
    fail = fail + int(mvl != 3) + int(smv != 5) + int(mvcells != 5)
    return fail

def test_double_polyhedral_homotopies(vrblvl=0):
    """
    Test the polyhedral homotopies in double precision
    to compute and solve a random coefficient system.
    The verbose level is given by vrblvl.
    """
    if vrblvl > 0:
        print('in test_double_polyhedral_homotopies ...')
    polynomials = ["x^3 + 2*x*y - 1;", "x + y - 1;"]
    mv = mixed_volume(polynomials, True, vrblvl)
    rndcffsys, sols = double_polyhedral_homotopies(vrblvl)
    fail = int(mv != len(sols))
    if vrblvl > 0:
        print('a random coefficient system :')
        for pol in rndcffsys:
            print(pol)
        print('its solutions :')
        for (idx, sol) in enumerate(sols):
            print('Solution', idx+1, ':')
            print(sol)
    err = verify(rndcffsys, sols, vrblvl-1)
    if vrblvl > 0:
        print('the sum of errors :', err)
    fail = fail + int(err > 1.0e-8)
    return fail

def test_double_double_polyhedral_homotopies(vrblvl=0):
    """
    Test the polyhedral homotopies in double double precision
    to compute and solve a random coefficient system.
    The verbose level is given by vrblvl.
    """
    if vrblvl > 0:
        print('in test_double_double_polyhedral_homotopies ...')
    polynomials = ["x^3 + 2*x*y - 1;", "x + y - 1;"]
    mv = mixed_volume(polynomials, True, vrblvl)
    rndcffsys, sols = double_double_polyhedral_homotopies(vrblvl)
    fail = int(mv != len(sols))
    if vrblvl > 0:
        print('a random coefficient system :')
        for pol in rndcffsys:
            print(pol)
        print('its solutions :')
        for (idx, sol) in enumerate(sols):
            print('Solution', idx+1, ':')
            print(sol)
    err = verify(rndcffsys, sols, vrblvl-1)
    if vrblvl > 0:
        print('the sum of errors :', err)
    fail = fail + int(err > 1.0e-8)
    return fail

def test_quad_double_polyhedral_homotopies(vrblvl=0):
    """
    Test the polyhedral homotopies in quad double precision
    to compute and solve a random coefficient system.
    The verbose level is given by vrblvl.
    """
    if vrblvl > 0:
        print('in test_quad_double_polyhedral_homotopies ...')
    polynomials = ["x^3 + 2*x*y - 1;", "x + y - 1;"]
    mv = mixed_volume(polynomials, True, vrblvl)
    rndcffsys, sols = quad_double_polyhedral_homotopies(vrblvl)
    fail = int(mv != len(sols))
    if vrblvl > 0:
        print('a random coefficient system :')
        for pol in rndcffsys:
            print(pol)
        print('its solutions :')
        for (idx, sol) in enumerate(sols):
            print('Solution', idx+1, ':')
            print(sol)
    err = verify(rndcffsys, sols, vrblvl-1)
    if vrblvl > 0:
        print('the sum of errors :', err)
    fail = fail + int(err > 1.0e-8)
    return fail

def test_make_random_coefficient_system(vrblvl=0):
    """
    Test the making of a random coefficient system.
    The verbose level is given by vrblvl.
    """
    if vrblvl > 0:
        print('in test_make_random_coefficient_system ...')
    pols = ["x^3 + 2*x*y - 1;", "x + y - 1;"]
    mv, rndcffsys, sols = make_random_coefficient_system(pols, vrblvl)
    fail = int(mv != len(sols))
    if vrblvl > 0:
        print('the mixed volume :', mv)
        print('a random coefficient system :')
        for pol in rndcffsys:
            print(pol)
        print('its solutions :')
        for (idx, sol) in enumerate(sols):
            print('Solution', idx+1, ':')
            print(sol)
    err = verify(rndcffsys, sols, vrblvl-1)
    if vrblvl > 0:
        print('the sum of errors :', err)
    fail = fail + int(err > 1.0e-8)
    return fail

def main():
    """
    Runs tests on mixed volumes and stable mixed volumes.
    """
    lvl = 1
    fail = test_mixed_volume(lvl)
    fail = fail + test_stable_mixed_volume(lvl)
    fail = fail + test_double_polyhedral_homotopies(lvl)
    fail = fail + test_double_double_polyhedral_homotopies(lvl)
    fail = fail + test_quad_double_polyhedral_homotopies(lvl)
    fail = fail + test_make_random_coefficient_system(lvl)
    if fail == 0:
        print('=> All tests passed.')
    else:
        print('Number of failed tests :', fail)

if __name__=='__main__':
    main()



"""
Given a witness set representation of a pure dimensional solution set,
the functions in this module separate the generic points in the witness set
according to the irreducible components of the solution set.
"""

def standard_decomposition(deg):
    """
    Returns the decomposition as a list of labels of witness points
    on the components, computed in standard double precision.
    """
    from phcpy.phcpy2c2 import py2c_factor_number_of_standard_components
    from phcpy.phcpy2c2 import py2c_factor_witness_points_of_standard_component
    from phcpy.phcpy2c2 import py2c_factor_standard_trace_sum_difference as stf
    nbcmp = py2c_factor_number_of_standard_components()
    result = []
    for i in range(1, nbcmp+1):
        compnt = py2c_factor_witness_points_of_standard_component(deg, i)
        tsd = stf(deg, i, len(compnt), compnt)
        result.append((eval(compnt), tsd))
    return result

def dobldobl_decomposition(deg):
    """
    Returns the decomposition as a list of labels of witness points
    on the components, computed in double double precision.
    """
    from phcpy.phcpy2c2 import py2c_factor_number_of_dobldobl_components
    from phcpy.phcpy2c2 import py2c_factor_witness_points_of_dobldobl_component
    from phcpy.phcpy2c2 import py2c_factor_dobldobl_trace_sum_difference as dtf
    nbcmp = py2c_factor_number_of_dobldobl_components()
    result = []
    for i in range(1, nbcmp+1):
        compnt = py2c_factor_witness_points_of_dobldobl_component(deg, i)
        tsd = dtf(deg, i, len(compnt), compnt)
        result.append((eval(compnt), tsd))
    return result

def quaddobl_decomposition(deg):
    """
    Returns the decomposition as a list of labels of witness points
    on the components, computed in quad double precision.
    """
    from phcpy.phcpy2c2 import py2c_factor_number_of_quaddobl_components
    from phcpy.phcpy2c2 import py2c_factor_witness_points_of_quaddobl_component
    from phcpy.phcpy2c2 import py2c_factor_quaddobl_trace_sum_difference as qtf
    nbcmp = py2c_factor_number_of_quaddobl_components()
    result = []
    for i in range(1, nbcmp+1):
        compnt = py2c_factor_witness_points_of_quaddobl_component(deg, i)
        tsd = qtf(deg, i, len(compnt), compnt)
        result.append((eval(compnt), tsd))
    return result

def decomposition(deg, precision='d'):
    """
    Returns the decomposition as a list of labels of witness points
    on the components, computed in precision 'd', 'dd', or 'qd',
    respectively for double, double double, or quad double.
    """
    if(precision == 'd'):
        return standard_decomposition(deg)
    elif(precision == 'dd'):
        return dobldobl_decomposition(deg)
    elif(precision == 'qd'):
        return quaddobl_decomposition(deg)
    else:
        print('wrong level of precision')
        return None

def standard_monodromy_breakup(embsys, esols, dim, \
    islaurent=0, verbose=True, nbloops=0):
    r"""
    Applies the monodromy breakup algorithm in standard double precision
    to factor the *dim*-dimensional algebraic set represented by the
    embedded system *embsys* and its solutions *esols*.
    If the embedded polynomial system is a Laurent system,
    then islaurent must equal one, the default is zero.
    If *verbose* is False, then no output is written.
    If *nbloops* equals zero, then the user is prompted to give
    the maximum number of loops.
    """
    from phcpy.phcpy2c2 import py2c_factor_set_standard_to_mute
    from phcpy.phcpy2c2 import py2c_factor_set_standard_to_verbose
    from phcpy.phcpy2c2 import py2c_factor_standard_assign_labels
    from phcpy.phcpy2c2 import py2c_factor_initialize_standard_monodromy
    from phcpy.phcpy2c2 import py2c_factor_initialize_standard_sampler
    from phcpy.phcpy2c2 import py2c_factor_initialize_standard_Laurent_sampler
    from phcpy.phcpy2c2 import py2c_factor_standard_trace_grid_diagnostics
    from phcpy.phcpy2c2 import py2c_factor_set_standard_trace_slice
    from phcpy.phcpy2c2 import py2c_factor_store_standard_gammas
    from phcpy.phcpy2c2 import py2c_factor_standard_track_paths
    from phcpy.phcpy2c2 import py2c_factor_store_standard_solutions
    from phcpy.phcpy2c2 import py2c_factor_restore_standard_solutions
    from phcpy.phcpy2c2 import py2c_factor_new_standard_slices
    from phcpy.phcpy2c2 import py2c_factor_swap_standard_slices
    from phcpy.phcpy2c2 import py2c_factor_permutation_after_standard_loop
    from phcpy.phcpy2c2 import py2c_factor_number_of_standard_components
    from phcpy.phcpy2c2 import py2c_factor_update_standard_decomposition
    from phcpy.phcpy2c2 import py2c_solcon_write_standard_solutions
    from phcpy.phcpy2c2 import py2c_solcon_clear_standard_solutions
    from phcpy.interface import store_standard_witness_set
    from phcpy.interface import store_standard_laurent_witness_set
    if(verbose):
        print('... applying monodromy factorization with standard doubles ...')
        py2c_factor_set_standard_to_verbose()
    else:
        py2c_factor_set_standard_to_mute()
    deg = len(esols)
    nvar = len(embsys)
    if(verbose):
        print('dim =', dim)
    if(islaurent == 1):
        store_standard_laurent_witness_set(nvar, dim, embsys, esols)
        py2c_factor_standard_assign_labels(nvar, deg)
        py2c_factor_initialize_standard_Laurent_sampler(dim)
    else:
        store_standard_witness_set(nvar, dim, embsys, esols)
        py2c_factor_standard_assign_labels(nvar, deg)
        py2c_factor_initialize_standard_sampler(dim)
    if(verbose):
        py2c_solcon_write_standard_solutions()
    if(nbloops == 0):
        strnbloops = input('give the maximum number of loops : ')
        nbloops = int(strnbloops)
    py2c_factor_initialize_standard_monodromy(nbloops, deg, dim)
    py2c_factor_store_standard_solutions()
    if(verbose):
        print('... initializing the grid in standard double precision ...')
    for i in range(1, 3):
        py2c_factor_set_standard_trace_slice(i)
        py2c_factor_store_standard_gammas(nvar)
        py2c_factor_standard_track_paths(islaurent)
        py2c_factor_store_standard_solutions()
        py2c_factor_restore_standard_solutions()
        py2c_factor_swap_standard_slices()
    (err, dis) = py2c_factor_standard_trace_grid_diagnostics()
    if(verbose):
        print('The diagnostics of the trace grid :')
        print('  largest error on the samples :', err)
        print('  smallest distance between the samples :', dis)
    for i in range(1, nbloops+1):
        if(verbose):
            print('... starting loop %d ...' % i)
        py2c_factor_new_standard_slices(dim, nvar)
        py2c_factor_store_standard_gammas(nvar)
        py2c_factor_standard_track_paths(islaurent)
        py2c_solcon_clear_standard_solutions()
        py2c_factor_store_standard_gammas(nvar)
        py2c_factor_standard_track_paths(islaurent)
        py2c_factor_store_standard_solutions()
        sprm = py2c_factor_permutation_after_standard_loop(deg)
        if(verbose):
            perm = eval(sprm)
            print('the permutation :', perm)
        nb0 = py2c_factor_number_of_standard_components()
        done = py2c_factor_update_standard_decomposition(deg, len(sprm), sprm)
        nb1 = py2c_factor_number_of_standard_components()
        if(verbose):
            print('number of factors : %d -> %d' % (nb0, nb1))
            deco = decomposition(deg)
            print('decomposition :', deco)
        if(done == 1):
            break
        py2c_factor_restore_standard_solutions()

def dobldobl_monodromy_breakup(embsys, esols, dim, \
    islaurent=0, verbose=True, nbloops=0):
    r"""
    Applies the monodromy breakup algorithm in double double precision
    to factor the *dim*-dimensional algebraic set represented by the embedded
    system *embsys* and its solutions *esols*.
    If the embedded polynomial system is a Laurent system,
    then islaurent must equal one, the default is zero.
    If *verbose* is False, then no output is written.
    If *nbloops* equals zero, then the user is prompted to give
    the maximum number of loops.
    """
    from phcpy.phcpy2c2 import py2c_factor_set_dobldobl_to_mute
    from phcpy.phcpy2c2 import py2c_factor_set_dobldobl_to_verbose
    from phcpy.phcpy2c2 import py2c_factor_dobldobl_assign_labels
    from phcpy.phcpy2c2 import py2c_factor_initialize_dobldobl_monodromy
    from phcpy.phcpy2c2 import py2c_factor_initialize_dobldobl_sampler
    from phcpy.phcpy2c2 import py2c_factor_initialize_dobldobl_Laurent_sampler
    from phcpy.phcpy2c2 import py2c_factor_dobldobl_trace_grid_diagnostics
    from phcpy.phcpy2c2 import py2c_factor_set_dobldobl_trace_slice
    from phcpy.phcpy2c2 import py2c_factor_store_dobldobl_gammas
    from phcpy.phcpy2c2 import py2c_factor_dobldobl_track_paths
    from phcpy.phcpy2c2 import py2c_factor_store_dobldobl_solutions
    from phcpy.phcpy2c2 import py2c_factor_restore_dobldobl_solutions
    from phcpy.phcpy2c2 import py2c_factor_new_dobldobl_slices
    from phcpy.phcpy2c2 import py2c_factor_swap_dobldobl_slices
    from phcpy.phcpy2c2 import py2c_factor_permutation_after_dobldobl_loop
    from phcpy.phcpy2c2 import py2c_factor_number_of_dobldobl_components
    from phcpy.phcpy2c2 import py2c_factor_update_dobldobl_decomposition
    from phcpy.phcpy2c2 import py2c_solcon_write_dobldobl_solutions
    from phcpy.phcpy2c2 import py2c_solcon_clear_dobldobl_solutions
    from phcpy.interface import store_dobldobl_solutions
    from phcpy.interface import store_dobldobl_system
    from phcpy.interface import store_dobldobl_laurent_system
    if(verbose):
        print('... applying monodromy factorization with double doubles ...')
        py2c_factor_set_dobldobl_to_verbose()
    else:
        py2c_factor_set_dobldobl_to_mute()
    deg = len(esols)
    nvar = len(embsys)
    if(verbose):
        print('nvar =', nvar, 'dim =', dim, 'deg =', deg)
    if(islaurent == 1):
        store_dobldobl_laurent_system(embsys)
        py2c_factor_dobldobl_assign_labels(nvar, deg)
        py2c_factor_initialize_dobldobl_Laurent_sampler(dim)
    else:
        store_dobldobl_system(embsys)
        py2c_factor_dobldobl_assign_labels(nvar, deg)
        py2c_factor_initialize_dobldobl_sampler(dim)
    if(verbose):
        py2c_solcon_write_dobldobl_solutions()
    if(nbloops == 0):
        strnbloops = input('give the maximum number of loops : ')
        nbloops = int(strnbloops)
    py2c_factor_initialize_dobldobl_monodromy(nbloops, deg, dim)
    py2c_factor_store_dobldobl_solutions()
    if(verbose):
        print('... initializing the grid ...')
    for i in range(1, 3):
        py2c_factor_set_dobldobl_trace_slice(i)
        py2c_factor_store_dobldobl_gammas(nvar)
        py2c_factor_dobldobl_track_paths(islaurent)
        py2c_factor_store_dobldobl_solutions()
        py2c_factor_restore_dobldobl_solutions()
        py2c_factor_swap_dobldobl_slices()
    (err, dis) = py2c_factor_dobldobl_trace_grid_diagnostics()
    if(verbose):
        print('The diagnostics of the trace grid :')
        print('  largest error on the samples :', err)
        print('  smallest distance between the samples :', dis)
    for i in range(1, nbloops+1):
        if(verbose):
            print('... starting loop %d ...' % i)
        py2c_factor_new_dobldobl_slices(dim, nvar)
        py2c_factor_store_dobldobl_gammas(nvar)
        py2c_factor_dobldobl_track_paths(islaurent)
        py2c_solcon_clear_dobldobl_solutions()
        py2c_factor_store_dobldobl_gammas(nvar)
        py2c_factor_dobldobl_track_paths(islaurent)
        py2c_factor_store_dobldobl_solutions()
        sprm = py2c_factor_permutation_after_dobldobl_loop(deg)
        if(verbose):
            perm = eval(sprm)
            print('the permutation :', perm)
        nb0 = py2c_factor_number_of_dobldobl_components()
        done = py2c_factor_update_dobldobl_decomposition(deg, len(sprm), sprm)
        nb1 = py2c_factor_number_of_dobldobl_components()
        if(verbose):
            print('number of factors : %d -> %d' % (nb0, nb1))
            deco = decomposition(deg, 'dd')
            print('decomposition :', deco)
        if(done == 1):
            break
        py2c_factor_restore_dobldobl_solutions()

def quaddobl_monodromy_breakup(embsys, esols, dim, \
    islaurent=0, verbose=True, nbloops=0):
    r"""
    Applies the monodromy breakup algorithm in quad double precision
    to factor the *dim*-dimensional algebraic set represented by the embedded
    system *embsys* and its solutions *esols*.
    If the embedded polynomial system is a Laurent system,
    then islaurent must equal one, the default is zero.
    If *verbose* is False, then no output is written.
    If *nbloops* equals zero, then the user is prompted to give
    the maximum number of loops.
    """
    from phcpy.phcpy2c2 import py2c_factor_set_quaddobl_to_mute
    from phcpy.phcpy2c2 import py2c_factor_set_quaddobl_to_verbose
    from phcpy.phcpy2c2 import py2c_factor_quaddobl_assign_labels
    from phcpy.phcpy2c2 import py2c_factor_initialize_quaddobl_monodromy
    from phcpy.phcpy2c2 import py2c_factor_initialize_quaddobl_sampler
    from phcpy.phcpy2c2 import py2c_factor_initialize_quaddobl_Laurent_sampler
    from phcpy.phcpy2c2 import py2c_factor_quaddobl_trace_grid_diagnostics
    from phcpy.phcpy2c2 import py2c_factor_set_quaddobl_trace_slice
    from phcpy.phcpy2c2 import py2c_factor_store_quaddobl_gammas
    from phcpy.phcpy2c2 import py2c_factor_quaddobl_track_paths
    from phcpy.phcpy2c2 import py2c_factor_store_quaddobl_solutions
    from phcpy.phcpy2c2 import py2c_factor_restore_quaddobl_solutions
    from phcpy.phcpy2c2 import py2c_factor_new_quaddobl_slices
    from phcpy.phcpy2c2 import py2c_factor_swap_quaddobl_slices
    from phcpy.phcpy2c2 import py2c_factor_permutation_after_quaddobl_loop
    from phcpy.phcpy2c2 import py2c_factor_number_of_quaddobl_components
    from phcpy.phcpy2c2 import py2c_factor_update_quaddobl_decomposition
    from phcpy.phcpy2c2 import py2c_solcon_write_quaddobl_solutions
    from phcpy.phcpy2c2 import py2c_solcon_clear_quaddobl_solutions
    from phcpy.interface import store_quaddobl_solutions
    from phcpy.interface import store_quaddobl_system
    from phcpy.interface import store_quaddobl_laurent_system
    if(verbose):
        print('... applying monodromy factorization with quad doubles ...')
        py2c_factor_set_quaddobl_to_verbose()
    else:
        py2c_factor_set_quaddobl_to_mute()
    deg = len(esols)
    nvar = len(embsys)
    if(verbose):
        print('dim =', dim)
    store_quaddobl_solutions(nvar, esols)
    if(islaurent == 1):
        store_quaddobl_laurent_system(embsys)
        py2c_factor_quaddobl_assign_labels(nvar, deg)
        py2c_factor_initialize_quaddobl_Laurent_sampler(dim)
    else:
        store_quaddobl_system(embsys)
        py2c_factor_quaddobl_assign_labels(nvar, deg)
        py2c_factor_initialize_quaddobl_sampler(dim)
    if(verbose):
        py2c_solcon_write_quaddobl_solutions()
    if(nbloops == 0):
        strnbloops = input('give the maximum number of loops : ')
        nbloops = int(strnbloops)
    py2c_factor_initialize_quaddobl_monodromy(nbloops, deg, dim)
    py2c_factor_store_quaddobl_solutions()
    if(verbose):
        print('... initializing the grid ...')
    for i in range(1, 3):
        py2c_factor_set_quaddobl_trace_slice(i)
        py2c_factor_store_quaddobl_gammas(nvar)
        py2c_factor_quaddobl_track_paths(islaurent)
        py2c_factor_store_quaddobl_solutions()
        py2c_factor_restore_quaddobl_solutions()
        py2c_factor_swap_quaddobl_slices()
    (err, dis) = py2c_factor_quaddobl_trace_grid_diagnostics()
    if(verbose):
        print('The diagnostics of the trace grid :')
        print('  largest error on the samples :', err)
        print('  smallest distance between the samples :', dis)
    for i in range(1, nbloops+1):
        if(verbose):
            print('... starting loop %d ...' % i)
        py2c_factor_new_quaddobl_slices(dim, nvar)
        py2c_factor_store_quaddobl_gammas(nvar)
        py2c_factor_quaddobl_track_paths(islaurent)
        py2c_solcon_clear_quaddobl_solutions()
        py2c_factor_store_quaddobl_gammas(nvar)
        py2c_factor_quaddobl_track_paths(islaurent)
        py2c_factor_store_quaddobl_solutions()
        sprm = py2c_factor_permutation_after_quaddobl_loop(deg)
        if(verbose):
            perm = eval(sprm)
            print('the permutation :', perm)
        nb0 = py2c_factor_number_of_quaddobl_components()
        done = py2c_factor_update_quaddobl_decomposition(deg, len(sprm), sprm)
        nb1 = py2c_factor_number_of_quaddobl_components()
        if(verbose):
            print('number of factors : %d -> %d' % (nb0, nb1))
            deco = decomposition(deg, 'qd')
            print('decomposition :', deco)
        if(done == 1):
            break
        py2c_factor_restore_quaddobl_solutions()

def monodromy_breakup(embsys, esols, dim, \
    islaurent=0, verbose=True, nbloops=0, prec='d'):
    r"""
    Applies the monodromy breakup algorithm to factor the *dim*-dimensional
    set represented by the embedded system *embsys* and its solutions *esols*.
    If the embedded polynomial system is a Laurent system,
    then islaurent must equal one, the default is zero.
    If *verbose* is False, then no output is written.
    If *nbloops* equals zero, then the user is prompted to give
    the maximum number of loops.
    Three different levels of *precision* are supported: double precision 'd'
    (for the value for prec) is the default, the two other precisions are
    double double precision 'dd' and quad double precision 'qd'.
    """
    if(prec == 'd'):
        standard_monodromy_breakup\
            (embsys, esols, dim, islaurent, verbose, nbloops)
    elif(prec == 'dd'):
        dobldobl_monodromy_breakup\
            (embsys, esols, dim, islaurent, verbose, nbloops)
    elif(prec == 'qd'):
        quaddobl_monodromy_breakup\
            (embsys, esols, dim, islaurent, verbose, nbloops)
    else:
        print('wrong argument for precision')

def factor(dim, witsys, witsols, \
    islaurent=0, verbose=True, nbloops=20, precision='d'):
    r"""
    Applies monodromy to factor an equidimensional algebraic set,
    given as a witness sets, with the embedded polynomials in *witsys*,
    and corresponding generic points in *witsols*.
    If the embedded polynomial system is a Laurent system,
    then islaurent must equal one, the default is zero.
    The dimension of the algebraic set is given in *dim*.
    The default *precision* is double 'd'.  Other valid values for precision
    are 'dd' for double double, or 'qd' for quad double.
    """
    if(precision == 'd'):
        standard_monodromy_breakup\
            (witsys, witsols, dim, islaurent, verbose, nbloops)
        return decomposition(len(witsols))
    elif(precision == 'dd'):
        dobldobl_monodromy_breakup\
            (witsys, witsols, dim, islaurent, verbose, nbloops)
        return decomposition(len(witsols), 'dd')
    elif(precision == 'qd'):
        quaddobl_monodromy_breakup\
            (witsys, witsols, dim, islaurent, verbose, nbloops)
        return decomposition(len(witsols), 'qd')
    else:
        print('wrong level of precision')
        return None

def decompose(deco, islaurent=0, verbose=True, nbloops=20, precision='d'):
    r"""
    Given in *deco* is a dictionary with as keys the dimension
    and as value a tuple with an embedded (Laurent) polynomial system
    and its corresponding solutions as the second item in the tuple.
    Each item in the dictionary is decomposed into irreducible factors.
    If the embedded polynomial system is a Laurent system,
    then islaurent must equal one, the default is zero.
    The default precision is double 'd'.  Other valid values for precision
    are 'dd' for double double, or 'qd' for quad double.
    Returns the dictionary deco, but each tuple (except for dimension 0)
    is extended with the partition of the generic points with the linear
    trace difference to represented the irreducible decomposition.
    """
    result = {}
    dims = list(deco.keys())
    dims.sort(reverse=True)
    for dim in dims:
        (pols, sols) = deco[dim]
        if(dim == 0):
            result[dim] = (pols, sols) # copy the isolated solutions
        else:
            factors = factor(dim, pols, sols, \
                islaurent, verbose, nbloops, precision)
            if verbose:
                print('the factorization at dimension', dim)
                print(factors)
            result[dim] = (pols, sols, factors)
    return result

def write_decomposition(deco):
    r"""
    Given in *deco* is a dictionary where the keys are dimensions.
    For each dimension, there is a tuple with a witness set representation
    of the solution set at that dimension.
    The decomposition in *deco* is written.
    """
    dims = list(deco.keys())
    dims.sort(reverse=True)
    for dim in dims:
        if dim > 0:
            (pols, sols, fact) = deco[dim]
            print 'the factorization at dimension', dim, \
                ' #components :', len(fact)
            print(fact)
        else:
            (pols, sols) = deco[dim]
            print 'the number of isolated solutions :', len(sols)

def solve(nvr, dim, pols, islaurent=False, \
    precision='d', tasks=0, nbloops=20, \
    tol=1.0e-8, rcotol=1.0e-6, evatol=1.0e-6, memtol=1.0e-6, verbose=True):
    r"""
    Computes a numerical irreducible decomposition for the polynomials
    in the list *pols*, where *nvr* is the number of variables in *pols*.
    The top dimension (the highest dimension of the solution set) is
    given in *dim* and could be by default set to *nvr*-1.
    If *islaurent*, then *pols* is considered a Laurent polynomial system
    and negative exponents may occur.
    The default precision is double 'd'.  Other valid values for precision
    are 'dd' for double double, or 'qd' for quad double.
    On return is a dictionary.  The keys in the dictionary are dimensions.
    For each dimension, a tuple represents a witness set.
    For dimension zero, the solution list contains the isolated solutions.
    For each nonzero dimension, the generic points in the witness set are
    partitioned according to the irreducible factors of the solution set
    at that dimension.
    """
    from phcpy.cascades import run_cascade
    deco = run_cascade(nvr, dim, pols, islaurent, \
               tol=1.0e-8, rcotol=rcotol, evatol=evatol, memtol=memtol, \
               tasks=tasks, prc=precision, verbose=verbose)
    fadc = decompose(deco, islaurent=int(islaurent), verbose=verbose, \
               nbloops=nbloops, precision=precision)
    if verbose:
        write_decomposition(fadc)
    return fadc

def standard_polysys_solve(pols, topdim=-1, \
    filter=True, factor=True, tasks=0, verbose=True):
    """
    Runs the cascades of homotopies on the polynomial system in pols
    in standard double precision.  The default top dimension topdim
    is the number of variables in pols minus one.
    """
    from phcpy.phcpy2c2 import py2c_standard_polysys_solve
    from phcpy.phcpy2c2 import py2c_copy_standard_polysys_witset
    from phcpy.solver import number_of_symbols
    from phcpy.interface import store_standard_system
    from phcpy.interface import load_standard_system, load_standard_solutions
    dim = number_of_symbols(pols)
    if(topdim == -1):
        topdim = dim - 1
    fail = store_standard_system(pols, nbvar=dim)
    fail = py2c_standard_polysys_solve(tasks,topdim, \
        int(filter),int(factor),int(verbose))
    witsols = []
    for soldim in range(0, topdim+1):
        fail = py2c_copy_standard_polysys_witset(soldim)
        witset = (load_standard_system(), load_standard_solutions())
        witsols.append(witset)
    return witsols

def standard_laursys_solve(pols, topdim=-1, \
    filter=True, factor=True, tasks=0, verbose=True):
    """
    Runs the cascades of homotopies on the Laurent polynomial system in pols
    in standard double precision.  The default top dimension topdim
    is the number of variables in pols minus one.
    """
    from phcpy.phcpy2c2 import py2c_standard_laursys_solve
    from phcpy.phcpy2c2 import py2c_copy_standard_laursys_witset
    from phcpy.solver import number_of_symbols
    from phcpy.interface import store_standard_laurent_system
    from phcpy.interface import load_standard_laurent_system
    from phcpy.interface import load_standard_solutions
    dim = number_of_symbols(pols)
    if(topdim == -1):
        topdim = dim - 1
    fail = store_standard_laurent_system(pols, nbvar=dim)
    fail = py2c_standard_laursys_solve(tasks,topdim, \
        int(filter),int(factor),int(verbose))
    witsols = []
    for soldim in range(0, topdim+1):
        fail = py2c_copy_standard_laursys_witset(soldim)
        witset = (load_standard_laurent_system(), load_standard_solutions())
        witsols.append(witset)
    return witsols

def dobldobl_polysys_solve(pols, topdim=-1, \
    filter=True, factor=True, tasks=0, verbose=True):
    """
    Runs the cascades of homotopies on the polynomial system in pols
    in double double precision.  The default top dimension topdim
    is the number of variables in pols minus one.
    """
    from phcpy.phcpy2c2 import py2c_dobldobl_polysys_solve
    from phcpy.phcpy2c2 import py2c_copy_dobldobl_polysys_witset
    from phcpy.solver import number_of_symbols
    from phcpy.interface import store_dobldobl_system
    from phcpy.interface import load_dobldobl_system, load_dobldobl_solutions
    dim = number_of_symbols(pols)
    if(topdim == -1):
        topdim = dim - 1
    fail = store_dobldobl_system(pols, nbvar=dim)
    fail = py2c_dobldobl_polysys_solve(tasks,topdim, \
        int(filter),int(factor),int(verbose))
    witsols = []
    for soldim in range(0, topdim+1):
        fail = py2c_copy_dobldobl_polysys_witset(soldim)
        witset = (load_dobldobl_system(), load_dobldobl_solutions())
        witsols.append(witset)
    return witsols

def dobldobl_laursys_solve(pols, topdim=-1, \
    filter=True, factor=True, tasks=0, verbose=True):
    """
    Runs the cascades of homotopies on the Laurent polynomial system in pols
    in double double precision.  The default top dimension topdim
    is the number of variables in pols minus one.
    """
    from phcpy.phcpy2c2 import py2c_dobldobl_laursys_solve
    from phcpy.phcpy2c2 import py2c_copy_dobldobl_laursys_witset
    from phcpy.solver import number_of_symbols
    from phcpy.interface import store_dobldobl_laurent_system
    from phcpy.interface import load_dobldobl_laurent_system
    from phcpy.interface import load_dobldobl_solutions
    dim = number_of_symbols(pols)
    if(topdim == -1):
        topdim = dim - 1
    fail = store_dobldobl_laurent_system(pols, nbvar=dim)
    fail = py2c_dobldobl_laursys_solve(tasks,topdim, \
        int(filter),int(factor),int(verbose))
    witsols = []
    for soldim in range(0, topdim+1):
        fail = py2c_copy_dobldobl_laursys_witset(soldim)
        witset = (load_dobldobl_laurent_system(), load_dobldobl_solutions())
        witsols.append(witset)
    return witsols

def quaddobl_polysys_solve(pols, topdim=-1, \
    filter=True, factor=True, tasks=0, verbose=True):
    """
    Runs the cascades of homotopies on the polynomial system in pols
    in quad double precision.  The default top dimension topdim
    is the number of variables in pols minus one.
    """
    from phcpy.phcpy2c2 import py2c_quaddobl_polysys_solve
    from phcpy.phcpy2c2 import py2c_copy_quaddobl_polysys_witset
    from phcpy.solver import number_of_symbols
    from phcpy.interface import store_quaddobl_system
    from phcpy.interface import load_quaddobl_system, load_quaddobl_solutions
    dim = number_of_symbols(pols)
    if(topdim == -1):
        topdim = dim - 1
    fail = store_quaddobl_system(pols, nbvar=dim)
    fail = py2c_quaddobl_polysys_solve(tasks,topdim, \
        int(filter),int(factor),int(verbose))
    witsols = []
    for soldim in range(0, topdim+1):
        fail = py2c_copy_quaddobl_polysys_witset(soldim)
        witset = (load_quaddobl_system(), load_quaddobl_solutions())
        witsols.append(witset)
    return witsols

def quaddobl_laursys_solve(pols, topdim=-1, \
    filter=True, factor=True, tasks=0, verbose=True):
    """
    Runs the cascades of homotopies on the Laurent polynomial system in pols
    in quad double precision.  The default top dimension topdim
    is the number of variables in pols minus one.
    """
    from phcpy.phcpy2c2 import py2c_quaddobl_laursys_solve
    from phcpy.phcpy2c2 import py2c_copy_quaddobl_laursys_witset
    from phcpy.solver import number_of_symbols
    from phcpy.interface import store_quaddobl_laurent_system
    from phcpy.interface import load_quaddobl_laurent_system
    from phcpy.interface import load_quaddobl_solutions
    dim = number_of_symbols(pols)
    if(topdim == -1):
        topdim = dim - 1
    fail = store_quaddobl_laurent_system(pols, nbvar=dim)
    fail = py2c_quaddobl_laursys_solve(tasks,topdim, \
        int(filter),int(factor),int(verbose))
    witsols = []
    for soldim in range(0, topdim+1):
        fail = py2c_copy_quaddobl_laursys_witset(soldim)
        witset = (load_quaddobl_laurent_system(), load_quaddobl_solutions())
        witsols.append(witset)
    return witsols

def polysys_solve(pols, topdim=-1, precision='d', \
    filter=True, factor=True, tasks=0, verbose=True):
    """
    Runs the cascades of homotopies on the polynomial system in pols
    in double, double double, or quad double precision.
    The default top dimension topdim is the number of variables 
    in pols minus one.
    """
    if(precision == 'd'):
        return standard_polysys_solve(pols, topdim, \
                                      filter, factor, tasks, verbose)
    elif(precision == 'dd'):
        return dobldobl_polysys_solve(pols, topdim, \
                                      filter, factor, tasks, verbose)
    elif(precision == 'qd'):
        return quaddobl_polysys_solve(pols, topdim, \
                                      filter, factor, tasks, verbose)
    else:
        print('wrong level of precision, use d, dd, or qd')

def laursys_solve(pols, topdim=-1, precision='d', \
    filter=True, factor=True, tasks=0, verbose=True):
    """
    Runs the cascades of homotopies on the Laurent polynomial system in pols
    in double, double double, or quad double precision.
    The default top dimension topdim is the number of variables 
    in pols minus one.
    """
    if(precision == 'd'):
        return standard_laursys_solve(pols, topdim, \
                                      filter, factor, tasks, verbose)
    elif(precision == 'dd'):
        return dobldobl_laursys_solve(pols, topdim, \
                                      filter, factor, tasks, verbose)
    elif(precision == 'qd'):
        return quaddobl_laursys_solve(pols, topdim, \
                                      filter, factor, tasks, verbose)
    else:
        print('wrong level of precision, use d, dd, or qd')

def test_monodromy(prc='d'):
    """
    Runs a test on applying monodromy loops
    to factor a curve into irreducible components.
    """
    from phcpy.solver import solve
    from phcpy.sets import embed
    pols = ['(x^2 - y)*(x-y);', 'x^3 - z;']
    embsys = embed(3, 1, pols, prc)
    # patch : make sure zz1 is last symbol!
    embsys[0] = 'x - x + y - y + z - z + ' + embsys[0]
    print(embsys)
    sols = solve(embsys, verbose=False, precision=prc)
    # for sol in sols: print sol
    print('the degree is', len(sols))
    monodromy_breakup(embsys, sols, 1, islaurent=0, verbose=True, prec=prc)

def test_factor():
    """
    Simple test on the factor method.
    """
    from phcpy.sets import witness_set_of_hypersurface
    hyp = '(x+1)*(x^2 + y^2 + 1);'
    (wsys, wsols) = witness_set_of_hypersurface(2, hyp)
    fac = factor(1, wsys, wsols)
    print(fac)

def test_decompose():
    """
    Runs a test on the decompose() function.
    """
    from cascades import run_cascade
    pols = ['(x1-1)*(x1-2)*(x1-3)*(x1-4);', \
            '(x1-1)*(x2-1)*(x2-2)*(x2-3);', \
            '(x1-1)*(x1-2)*(x3-1)*(x3-2);', \
            '(x1-1)*(x2-1)*(x3-1)*(x4-1);']
    deco = run_cascade(4, 3, pols)
    fadc = decompose(deco)
    write_decomposition(fadc)

def test_solve():
    """
    Runs a test on the solve() function.
    """
    testpols = ['(x1-1)*(x1-2)*(x1-3)*(x1-4);', \
                '(x1-1)*(x2-1)*(x2-2)*(x2-3);', \
                '(x1-1)*(x1-2)*(x3-1)*(x3-2);', \
                '(x1-1)*(x2-1)*(x3-1)*(x4-1);']
    ans = raw_input('Verbose ? (y/n) ')
    vrb = (ans == 'y')
    deco = solve(nvr=4, dim=3, pols=testpols, verbose=vrb)
    if not vrb:
        print 'the decomposition :'
        write_decomposition(deco)

def test_polysys_solve():
    """
    Runs a test on the standard_polysys_solve() function.
    """
    pols = ['(x1-1)*(x1-2)*(x1-3)*(x1-4);', \
            '(x1-1)*(x2-1)*(x2-2)*(x2-3);', \
            '(x1-1)*(x1-2)*(x3-1)*(x3-2);', \
            '(x1-1)*(x2-1)*(x3-1)*(x4-1);']
    # sols = standard_polysys_solve(pols)
    sols = dobldobl_polysys_solve(pols)
    for dim in range(0, len(sols)):
        witset = sols[dim]
        deg = len(witset[1])
        print('degree of solution set at dimension', dim, ':', deg)

def test():
    """
    Sets the seed for the random number generators
    to a fixed number and then runs a test.
    """
    from phcpy.phcpy2c2 import py2c_set_seed
    py2c_set_seed(234798272)
    # test_monodromy()
    # test_decompose()
    test_solve()

if __name__ == "__main__":
    test()



"""
To install phcpy correctly, the shared object file must be copied
to the same location where the modules of phcpy are installed.
This is accomplished via the data_files entry in the setup below.
For the installation of the GPU accelerated path trackers in double,
double double and quad double precision, instead of phcpy2c3.so,
the shared objects that are needed are phcpy2c3path_d.so (double),
phcpy2c3path_dd.so (double double), and phcpy2c3path_qd.so (quad double).
"""

from distutils.core import setup

setup(
    name = 'PHCpy' ,
    author = 'Jan Verschelde' ,
    author_email = 'janv@uic.edu' ,
    description = 'a package for Polynomial Homotopy Continuation' ,
    url = 'https://github.com/janverschelde/PHCpack' ,
    version = '1.1.2' ,
    packages = ['phcpy'] ,
    py_modules = ['phcpy/interface', 'phcpy/solver', \
        'phcpy/solutions', 'phcpy/polynomials', \
        'phcpy/trackers', 'phcpy/sweepers', 'phcpy/tuning', \
        'phcpy/sets', 'phcpy/cascades', 'phcpy/factor', 'phcpy/diagonal', \
        'phcpy/polytopes', 'phcpy/tropisms', 'phcpy/maps', 'phcpy/series', \
        'phcpy/curves', 'phcpy/examples', 'phcpy/families', \
        'phcpy/schubert' , 'phcpy/dashboard', 'phcpy/server' ],
    license = 'GNU GENERAL PUBLIC LICENSE version 3' ,
    package_data = {'phcpy':['phcpy2c3.so']} ,
    platforms = ['linux2'] ,
    long_description=open('README.txt').read()
)



"""
Given two witness sets for two pure dimensional solution sets,
a diagonal homotopy computes a sets of witness sets for all components
of the intersection of the two pure dimensional solution sets.
"""

def top_diagonal_dimension(kdm, dim1, dim2):
    r"""
    Returns the number of slack variables at the top in the cascade of
    diagonal homotopies to intersect two sets of dimension *dim1* and *dim2*,
    where *dim1* >= *dim2* and *kdm* is the dimension before the embedding.
    Typically, *kdm* is the number of equations in the first witness set
    minus *dim1*.
    """
    if dim1 + dim2 < kdm:
        return dim2
    else:
        return kdm - dim1

def standard_diagonal_homotopy(dim1, sys1, esols1, dim2, sys2, esols2):
    r"""
    Defines a diagonal homotopy to intersect the witness sets defined
    by (*sys1*, *esols1*) and (*sys2*, *esols2*), respectively of dimensions
    *dim1* and *dim2*.  The systems *sys1* and *sys2* are assumed to be square
    and with as many slack variables as the dimension of the solution sets.
    The data is stored in standard double precision.
    """
    from phcpy.interface import store_standard_system as storesys
    from phcpy.interface import store_standard_solutions as storesols
    from phcpy.phcpy2c3 import py2c_copy_standard_container_to_target_system
    from phcpy.phcpy2c3 import py2c_copy_standard_container_to_target_solutions
    from phcpy.phcpy2c3 import py2c_copy_standard_container_to_start_system
    from phcpy.phcpy2c3 import py2c_copy_standard_container_to_start_solutions
    from phcpy.phcpy2c3 import py2c_standard_diagonal_homotopy
    from phcpy.phcpy2c3 import py2c_syscon_number_of_symbols
    from phcpy.phcpy2c3 import py2c_syscon_string_of_symbols
    from phcpy.phcpy2c3 import py2c_diagonal_symbols_doubler
    storesys(sys1)
    symbols = py2c_syscon_string_of_symbols()
    nbsymbs = py2c_syscon_number_of_symbols()
    print('number of symbols :', nbsymbs)
    print('names of variables :', symbols)
    storesols(len(sys1), esols1)
    if(dim1 >= dim2):
        py2c_copy_standard_container_to_target_system()
        py2c_copy_standard_container_to_target_solutions()
    else:
        py2c_copy_standard_container_to_start_system()
        py2c_copy_standard_container_to_start_solutions()
    storesys(sys2)
    storesols(len(sys2), esols2)
    if(dim1 >= dim2):
        py2c_copy_standard_container_to_start_system()
        py2c_copy_standard_container_to_start_solutions()
    else:
        py2c_copy_standard_container_to_target_system()
        py2c_copy_standard_container_to_target_solutions()
    if(dim1 >= dim2):
        py2c_standard_diagonal_homotopy(dim1, dim2)
    else:
        py2c_standard_diagonal_homotopy(dim2, dim1)
    py2c_diagonal_symbols_doubler(nbsymbs-dim1, dim1, len(symbols), symbols)

def dobldobl_diagonal_homotopy(dim1, sys1, esols1, dim2, sys2, esols2):
    r"""
    Defines a diagonal homotopy to intersect the witness sets defined
    by (*sys1*, *esols1*) and (*sys2*, *esols2*), respectively of dimensions
    *dim1* and *dim2*.  The systems *sys1* and *sys2* are assumed to be square
    and with as many slack variables as the dimension of the solution sets.
    The data is stored in double double precision.
    """
    from phcpy.interface import store_dobldobl_system as storesys
    from phcpy.interface import store_dobldobl_solutions as storesols
    from phcpy.phcpy2c3 import py2c_copy_dobldobl_container_to_target_system
    from phcpy.phcpy2c3 import py2c_copy_dobldobl_container_to_target_solutions
    from phcpy.phcpy2c3 import py2c_copy_dobldobl_container_to_start_system
    from phcpy.phcpy2c3 import py2c_copy_dobldobl_container_to_start_solutions
    from phcpy.phcpy2c3 import py2c_dobldobl_diagonal_homotopy
    from phcpy.phcpy2c3 import py2c_syscon_number_of_symbols
    from phcpy.phcpy2c3 import py2c_syscon_string_of_symbols
    from phcpy.phcpy2c3 import py2c_diagonal_symbols_doubler
    storesys(sys1)
    symbols = py2c_syscon_string_of_symbols()
    nbsymbs = py2c_syscon_number_of_symbols()
    print('number of symbols :', nbsymbs)
    print('names of variables :', symbols)
    storesols(len(sys1), esols1)
    if(dim1 >= dim2):
        py2c_copy_dobldobl_container_to_target_system()
        py2c_copy_dobldobl_container_to_target_solutions()
    else:
        py2c_copy_dobldobl_container_to_start_system()
        py2c_copy_dobldobl_container_to_start_solutions()
    storesys(sys2)
    storesols(len(sys2), esols2)
    if(dim1 >= dim2):
        py2c_copy_dobldobl_container_to_start_system()
        py2c_copy_dobldobl_container_to_start_solutions()
    else:
        py2c_copy_dobldobl_container_to_target_system()
        py2c_copy_dobldobl_container_to_target_solutions()
    if(dim1 >= dim2):
        py2c_dobldobl_diagonal_homotopy(dim1, dim2)
    else:
        py2c_dobldobl_diagonal_homotopy(dim2, dim1)
    py2c_diagonal_symbols_doubler(nbsymbs-dim1, dim1, len(symbols), symbols)

def quaddobl_diagonal_homotopy(dim1, sys1, esols1, dim2, sys2, esols2):
    r"""
    Defines a diagonal homotopy to intersect the witness sets defined
    by (*sys1*, *esols1*) and (*sys2*, *esols2*), respectively of dimensions
    *dim1* and *dim2*.  The systems *sys1* and *sys2* are assumed to be square
    and with as many slack variables as the dimension of the solution sets.
    The data is stored in quad double precision.
    """
    from phcpy.interface import store_quaddobl_system as storesys
    from phcpy.interface import store_quaddobl_solutions as storesols
    from phcpy.phcpy2c3 import py2c_copy_quaddobl_container_to_target_system
    from phcpy.phcpy2c3 import py2c_copy_quaddobl_container_to_target_solutions
    from phcpy.phcpy2c3 import py2c_copy_quaddobl_container_to_start_system
    from phcpy.phcpy2c3 import py2c_copy_quaddobl_container_to_start_solutions
    from phcpy.phcpy2c3 import py2c_quaddobl_diagonal_homotopy
    from phcpy.phcpy2c3 import py2c_syscon_number_of_symbols
    from phcpy.phcpy2c3 import py2c_syscon_string_of_symbols
    from phcpy.phcpy2c3 import py2c_diagonal_symbols_doubler
    storesys(sys1)
    symbols = py2c_syscon_string_of_symbols()
    nbsymbs = py2c_syscon_number_of_symbols()
    print('number of symbols :', nbsymbs)
    print('names of variables :', symbols)
    storesols(len(sys1), esols1)
    if(dim1 >= dim2):
        py2c_copy_quaddobl_container_to_target_system()
        py2c_copy_quaddobl_container_to_target_solutions()
    else:
        py2c_copy_quaddobl_container_to_start_system()
        py2c_copy_quaddobl_container_to_start_solutions()
    storesys(sys2)
    storesols(len(sys2), esols2)
    if(dim1 >= dim2):
        py2c_copy_quaddobl_container_to_start_system()
        py2c_copy_quaddobl_container_to_start_solutions()
    else:
        py2c_copy_quaddobl_container_to_target_system()
        py2c_copy_quaddobl_container_to_target_solutions()
    if(dim1 >= dim2):
        py2c_quaddobl_diagonal_homotopy(dim1, dim2)
    else:
        py2c_quaddobl_diagonal_homotopy(dim2, dim1)
    py2c_diagonal_symbols_doubler(nbsymbs-dim1, dim1, len(symbols), symbols)

def standard_diagonal_cascade_solutions(dim1, dim2):
    """
    Defines the start solutions in the cascade to start the diagonal
    homotopy to intersect a set of dimension *dim1* with another set
    of dimension *dim2*, in standard double precision.  For this to work,
    standard_diagonal_homotopy must have been executed successfully.
    """
    from phcpy.phcpy2c3 import py2c_standard_diagonal_cascade_solutions
    if(dim1 >= dim2):
        py2c_standard_diagonal_cascade_solutions(dim1, dim2)
    else:
        py2c_standard_diagonal_cascade_solutions(dim2, dim1)

def dobldobl_diagonal_cascade_solutions(dim1, dim2):
    r"""
    Defines the start solutions in the cascade to start the diagonal
    homotopy to intersect a set of dimension *dim1* with another set
    of dimension *dim2*, in double double precision.  For this to work,
    dobldobl_diagonal_homotopy must have been executed successfully.
    """
    from phcpy.phcpy2c3 import py2c_dobldobl_diagonal_cascade_solutions
    if(dim1 >= dim2):
        py2c_dobldobl_diagonal_cascade_solutions(dim1, dim2)
    else:
        py2c_dobldobl_diagonal_cascade_solutions(dim2, dim1)

def quaddobl_diagonal_cascade_solutions(dim1, dim2):
    r"""
    Defines the start solutions in the cascade to start the diagonal
    homotopy to intersect a set of dimension *dim1* with another set
    of dimension *dim2*, in quad double precision.  For this to work,
    quaddobl_diagonal_homotopy must have been executed successfully.
    """
    from phcpy.phcpy2c3 import py2c_quaddobl_diagonal_cascade_solutions
    if(dim1 >= dim2):
        py2c_quaddobl_diagonal_cascade_solutions(dim1, dim2)
    else:
        py2c_quaddobl_diagonal_cascade_solutions(dim2, dim1)

def standard_start_diagonal_cascade(gamma=0, tasks=0):
    r"""
    Does the path tracking to start a diagonal cascade in standard double
    precision.  For this to work, the functions standard_diagonal_homotopy
    and standard_diagonal_cascade_solutions must be executed successfully.
    If *gamma* equals 0 on input, then a random gamma constant is generated,
    otherwise, the given complex gamma will be used in the homotopy.
    Multitasking is available, and activated by the *tasks* parameter.
    Returns the target (system and its corresponding) solutions.
    """
    from phcpy.phcpy2c3 import py2c_create_standard_homotopy
    from phcpy.phcpy2c3 import py2c_create_standard_homotopy_with_gamma
    from phcpy.phcpy2c3 import py2c_solve_by_standard_homotopy_continuation
    from phcpy.phcpy2c3 import py2c_solcon_clear_standard_solutions
    from phcpy.phcpy2c3 import py2c_syscon_clear_standard_system
    from phcpy.phcpy2c3 import py2c_copy_standard_target_solutions_to_container
    from phcpy.phcpy2c3 import py2c_copy_standard_target_system_to_container
    from phcpy.interface import load_standard_solutions
    from phcpy.interface import load_standard_system
    if(gamma == 0):
        py2c_create_standard_homotopy()
    else:
        py2c_create_standard_homotopy_with_gamma(gamma.real, gamma.imag)
    py2c_solve_by_standard_homotopy_continuation(tasks)
    py2c_solcon_clear_standard_solutions()
    py2c_syscon_clear_standard_system()
    py2c_copy_standard_target_solutions_to_container()
    # from phcpy.phcpy2c3 import py2c_write_standard_target_system
    # print 'the standard target system :'
    # py2c_write_standard_target_system()
    py2c_copy_standard_target_system_to_container()
    tsys = load_standard_system()
    sols = load_standard_solutions()
    return (tsys, sols)

def dobldobl_start_diagonal_cascade(gamma=0, tasks=0):
    r"""
    Does the path tracking to start a diagonal cascade in double double
    precision.  For this to work, the functions dobldobl_diagonal_homotopy
    and dobldobl_diagonal_cascade_solutions must be executed successfully.
    If *gamma* equals 0 on input, then a random gamma constant is generated,
    otherwise, the given complex gamma will be used in the homotopy.
    Multitasking is available, and activated by the *tasks* parameter.
    Returns the target (system and its corresponding) solutions.
    """
    from phcpy.phcpy2c3 import py2c_create_dobldobl_homotopy
    from phcpy.phcpy2c3 import py2c_create_dobldobl_homotopy_with_gamma
    from phcpy.phcpy2c3 import py2c_solve_by_dobldobl_homotopy_continuation
    from phcpy.phcpy2c3 import py2c_solcon_clear_dobldobl_solutions
    from phcpy.phcpy2c3 import py2c_syscon_clear_dobldobl_system
    from phcpy.phcpy2c3 import py2c_copy_dobldobl_target_solutions_to_container
    from phcpy.phcpy2c3 import py2c_copy_dobldobl_target_system_to_container
    from phcpy.interface import load_dobldobl_solutions
    from phcpy.interface import load_dobldobl_system
    if(gamma == 0):
        py2c_create_dobldobl_homotopy()
    else:
        py2c_create_dobldobl_homotopy_with_gamma(gamma.real, gamma.imag)
    py2c_solve_by_dobldobl_homotopy_continuation(tasks)
    py2c_solcon_clear_dobldobl_solutions()
    py2c_syscon_clear_dobldobl_system()
    py2c_copy_dobldobl_target_solutions_to_container()
    # from phcpy.phcpy2c3 import py2c_write_dobldobl_target_system
    # print 'the dobldobl target system :'
    # py2c_write_dobldobl_target_system()
    py2c_copy_dobldobl_target_system_to_container()
    tsys = load_dobldobl_system()
    sols = load_dobldobl_solutions()
    return (tsys, sols)

def quaddobl_start_diagonal_cascade(gamma=0, tasks=0):
    r"""
    Does the path tracking to start a diagonal cascade in quad double
    precision.  For this to work, the functions quaddobl_diagonal_homotopy
    and quaddobl_diagonal_cascade_solutions must be executed successfully.
    If *gamma* equals 0 on input, then a random gamma constant is generated,
    otherwise, the given complex gamma will be used in the homotopy.
    Multitasking is available, and is activated by the *tasks* parameter.
    Returns the target (system and its corresponding) solutions.
    """
    from phcpy.phcpy2c3 import py2c_create_quaddobl_homotopy
    from phcpy.phcpy2c3 import py2c_create_quaddobl_homotopy_with_gamma
    from phcpy.phcpy2c3 import py2c_solve_by_quaddobl_homotopy_continuation
    from phcpy.phcpy2c3 import py2c_solcon_clear_quaddobl_solutions
    from phcpy.phcpy2c3 import py2c_syscon_clear_quaddobl_system
    from phcpy.phcpy2c3 import py2c_copy_quaddobl_target_solutions_to_container
    from phcpy.phcpy2c3 import py2c_copy_quaddobl_target_system_to_container
    from phcpy.interface import load_quaddobl_solutions
    from phcpy.interface import load_quaddobl_system
    if(gamma == 0):
        py2c_create_quaddobl_homotopy()
    else:
        py2c_create_quaddobl_homotopy_with_gamma(gamma.real, gamma.imag)
    py2c_solve_by_quaddobl_homotopy_continuation(tasks)
    py2c_solcon_clear_quaddobl_solutions()
    py2c_syscon_clear_quaddobl_system()
    py2c_copy_quaddobl_target_solutions_to_container()
    # from phcpy.phcpy2c3 import py2c_write_quaddobl_target_system
    # print 'the quaddobl target system :'
    # py2c_write_quaddobl_target_system()
    py2c_copy_quaddobl_target_system_to_container()
    tsys = load_quaddobl_system()
    sols = load_quaddobl_solutions()
    return (tsys, sols)

def standard_diagonal_solver(dim, dm1, sys1, sols1, dm2, sys2, sols2,
    tasks=0, verbose=True):
    r"""
    Runs the diagonal homotopies in standard double precision
    to intersect two witness sets stored in (*sys1*, *sols1*) and
    (*sys2*, *sols2*), of respective dimensions *dm1* and *dm2*.
    The ambient dimension equals *dim*.
    Multitasking is available, and is activated by the *tasks* parameter.
    Returns the last system in the cascade and its solutions.
    If *verbose*, then the solver runs in interactive mode, printing
    intermediate results to screen and prompting the user to continue.
    """
    from phcpy.phcpy2c3 import py2c_standard_collapse_diagonal
    from phcpy.interface import store_standard_solutions as storesols
    from phcpy.interface import load_standard_solutions as loadsols
    from phcpy.interface import load_standard_system as loadsys
    from phcpy.phcpy2c3 import py2c_extrinsic_top_diagonal_dimension
    from phcpy.solutions import filter_vanishing
    from phcpy.sets import drop_coordinate_from_standard_solutions
    from phcpy.sets import drop_variable_from_standard_polynomials
    from phcpy.cascades import standard_double_cascade_step
    topdim = py2c_extrinsic_top_diagonal_dimension(dim+dm1, dim+dm2, dm1, dm2)
    kdm = len(sys1) - dm1
    topdiagdim = top_diagonal_dimension(kdm, dm1, dm2)
    if verbose:
        print('the top dimension :', topdim, 'dim :', dim)
        print('number of slack variables at the top :', topdiagdim)
    standard_diagonal_homotopy(dm1, sys1, sols1, dm2, sys2, sols2)
    if verbose:
        print('defining the start solutions')
    standard_diagonal_cascade_solutions(dm1, dm2)
    if verbose:
        print('starting the diagonal cascade')
    (topsys, startsols) = standard_start_diagonal_cascade()
    if verbose:
        print('the system solved in the start of the cascade :')
        for pol in topsys:
            print(pol)
        print('the solutions after starting the diagonal cascade :')
        for sol in startsols:
            print(sol)
        input('hit enter to continue')
    for k in range(topdiagdim, 0, -1):
        endsols = standard_double_cascade_step(k, topsys, startsols)
        if verbose:
            print('after running cascade step %d :' % k)
            for sol in endsols:
                print(sol)
        endsolsf1 = filter_vanishing(endsols, 1.0e-8)
        if verbose:
            print('computed', len(endsolsf1), 'solutions')
            input('hit enter to continue')
        slack = 'zz' + str(k)
        nbvar = len(topsys)
        endsolsf2 = drop_coordinate_from_standard_solutions\
            (endsolsf1, nbvar, slack)
        if verbose:
            print('after dropping the slack coordinate from the solutions :')
            for sol in endsolsf2:
                print(sol)
            input('hit enter to continue')
        nextsys = drop_variable_from_standard_polynomials(topsys, slack)
        if verbose:
            print('after dropping the variable', slack, 'from the system :')
            for pol in nextsys:
                print(pol)
        (topsys, startsols) = (nextsys[:-1], endsolsf2)
    storesols(len(topsys), startsols)
    # py2c_standard_collapse_diagonal(topdim - 2*dim, 0)
    py2c_standard_collapse_diagonal(0, 0)
    result = (loadsys(), loadsols())
    return result

def dobldobl_diagonal_solver(dim, dm1, sys1, sols1, dm2, sys2, sols2, \
    tasks=0, verbose=True):
    r"""
    Runs the diagonal homotopies in double double precision
    to intersect two witness sets stored in (*sys1*, *sols1*) and
    (*sys2*, *sols2*), of respective dimensions *dm1* and *dm2*.
    The ambient dimension equals *dim*.
    Multitasking is available, and is activated by the *tasks* parameter.
    Returns the last system in the cascade and its solutions.
    If *verbose*, then the solver runs in interactive mode, printing
    intermediate results to screen and prompting the user to continue.
    """
    from phcpy.phcpy2c3 import py2c_dobldobl_collapse_diagonal
    from phcpy.interface import store_dobldobl_solutions as storesols
    from phcpy.interface import load_dobldobl_solutions as loadsols
    from phcpy.interface import load_dobldobl_system as loadsys
    from phcpy.phcpy2c3 import py2c_extrinsic_top_diagonal_dimension
    from phcpy.solutions import filter_vanishing
    from phcpy.sets import drop_coordinate_from_dobldobl_solutions
    from phcpy.sets import drop_variable_from_dobldobl_polynomials
    from phcpy.cascades import double_double_cascade_step
    topdim = py2c_extrinsic_top_diagonal_dimension(dim+dm1, dim+dm2, dm1, dm2)
    kdm = len(sys1) - dm1
    topdiagdim = top_diagonal_dimension(kdm, dm1, dm2)
    if verbose:
        print('the top dimension :', topdim, 'dim :', dim)
        print('number of slack variables at the top :', topdiagdim)
    dobldobl_diagonal_homotopy(dm1, sys1, sols1, dm2, sys2, sols2)
    if verbose:
        print('defining the start solutions')
    dobldobl_diagonal_cascade_solutions(dm1, dm2)
    if verbose:
        print('starting the diagonal cascade')
    (topsys, startsols) = dobldobl_start_diagonal_cascade()
    if verbose:
        print('the system solved in the start of the cascade :')
        for pol in topsys:
            print(pol)
        print('the solutions after starting the diagonal cascade :')
        for sol in startsols:
            print(sol)
        input('hit enter to continue')
    for k in range(topdiagdim, 0, -1):
        endsols = double_double_cascade_step(k, topsys, startsols)
        if verbose:
            print('after running cascade step %d :' % k)
            for sol in endsols:
                print(sol)
        endsolsf1 = filter_vanishing(endsols, 1.0e-8)
        if verbose:
            print('computed', len(endsolsf1), 'solutions')
            input('hit enter to continue')
        slack = 'zz' + str(k)
        nbvar = len(topsys)
        endsolsf2 = drop_coordinate_from_dobldobl_solutions\
            (endsolsf1, nbvar, slack)
        if verbose:
            print('after dropping the slack coordinate from the solutions :')
            for sol in endsolsf2:
                print(sol)
            input('hit enter to continue')
        nextsys = drop_variable_from_dobldobl_polynomials(topsys, slack)
        if verbose:
            print('after dropping the variable', slack, 'from the system :')
            for pol in nextsys:
                print(pol)
        (topsys, startsols) = (nextsys[:-1], endsolsf2)
    storesols(len(topsys), startsols)
    # py2c_dobldobl_collapse_diagonal(topdim - 2*dim, 0)
    py2c_dobldobl_collapse_diagonal(0, 0)
    result = (loadsys(), loadsols())
    return result

def quaddobl_diagonal_solver(dim, dm1, sys1, sols1, dm2, sys2, sols2, \
    tasks=0, verbose=True):
    r"""
    Runs the diagonal homotopies in quad double precision
    to intersect two witness sets stored in (*sys1*, *sols1*) and
    (*sys2*, *sols2*), of respective dimensions *dm1* and *dm2*.
    The ambient dimension equals *dim*.
    Multitasking is available, and is activated by the *tasks* parameter.
    Returns the last system in the cascade and its solutions.
    If *verbose*, then the solver runs in interactive mode, printing
    intermediate results to screen and prompting the user to continue.
    """
    from phcpy.phcpy2c3 import py2c_quaddobl_collapse_diagonal
    from phcpy.interface import store_quaddobl_solutions as storesols
    from phcpy.interface import load_quaddobl_solutions as loadsols
    from phcpy.interface import load_quaddobl_system as loadsys
    from phcpy.phcpy2c3 import py2c_extrinsic_top_diagonal_dimension
    from phcpy.solutions import filter_vanishing
    from phcpy.sets import drop_coordinate_from_quaddobl_solutions
    from phcpy.sets import drop_variable_from_quaddobl_polynomials
    from phcpy.cascades import quad_double_cascade_step
    topdim = py2c_extrinsic_top_diagonal_dimension(dim+dm1, dim+dm2, dm1, dm2)
    kdm = len(sys1) - dm1
    topdiagdim = top_diagonal_dimension(kdm, dm1, dm2)
    if verbose:
        print('the top dimension :', topdim, 'dim :', dim)
        print('number of slack variables at the top :', topdiagdim)
    quaddobl_diagonal_homotopy(dm1, sys1, sols1, dm2, sys2, sols2)
    if verbose:
        print('defining the start solutions')
    quaddobl_diagonal_cascade_solutions(dm1, dm2)
    if verbose:
        print('starting the diagonal cascade')
    (topsys, startsols) = quaddobl_start_diagonal_cascade()
    if verbose:
        for pol in topsys:
            print(pol)
        print('the solutions after starting the diagonal cascade :')
        for sol in startsols:
            print(sol)
        input('hit enter to continue')
    for k in range(topdiagdim, 0, -1):
        endsols = quad_double_cascade_step(k, topsys, startsols)
        if verbose:
            print('after running cascade step %d :' % k)
            for sol in endsols:
                print(sol)
        endsolsf1 = filter_vanishing(endsols, 1.0e-8)
        if verbose:
            print('computed', len(endsolsf1), 'solutions')
            input('hit enter to continue')
        slack = 'zz' + str(k)
        nbvar = len(topsys)
        endsolsf2 = drop_coordinate_from_quaddobl_solutions\
            (endsolsf1, nbvar, slack)
        if verbose:
            print('after dropping the slack coordinate from the solutions :')
            for sol in endsolsf2:
                print(sol)
            input('hit enter to continue')
        nextsys = drop_variable_from_quaddobl_polynomials(topsys, slack)
        if verbose:
            print('after dropping the variable', slack, 'from the system :')
            for pol in nextsys:
                print(pol)
        (topsys, startsols) = (nextsys[:-1], endsolsf2)
    storesols(len(topsys), startsols)
    # py2c_quaddobl_collapse_diagonal(topdim - 2*dim, 0)
    py2c_quaddobl_collapse_diagonal(0, 0)
    result = (loadsys(), loadsols())
    return result

def diagonal_solver(dim, dm1, sys1, sols1, dm2, sys2, sols2, tasks=0, \
    prc='d', verbose=True):
    r"""
    Runs the diagonal homotopies to intersect two witness sets stored in
    (*sys1*, *sols1*) and (*sys2*, *sols2*), of respective dimensions *dim1*
    and *dim2*.  The ambient dimension equals *dim*.
    Multitasking is available, and is activated by the *tasks* parameter.
    The precision is set by the parameter *prc*, which takes the default
    value 'd' for standard double, 'dd' for double double, or 'qd' for
    quad double precision.
    Returns the last system in the cascade and its solutions.
    """
    if(prc == 'd'):
        return standard_diagonal_solver\
                   (dim, dm1, sys1, sols1, dm2, sys2, sols2, tasks, verbose)
    elif(prc == 'dd'):
        return dobldobl_diagonal_solver\
                   (dim, dm1, sys1, sols1, dm2, sys2, sols2, tasks, verbose)
    elif(prc == 'qd'):
        return quaddobl_diagonal_solver\
                   (dim, dm1, sys1, sols1, dm2, sys2, sols2, tasks, verbose)
    else:
        print('wrong argument for precision')
        return None

def test_diaghom(precision='d'):
    """
    Test on the diagonal homotopy.
    """
    from phcpy.sets import witness_set_of_hypersurface
    hyp1 = 'x1*x2;'
    hyp2 = 'x1 - x2;'
    (w1sys, w1sols) = witness_set_of_hypersurface(2, hyp1, precision)
    print('the witness sets for', hyp1)
    for pol in w1sys:
        print(pol)
    for sol in w1sols:
        print(sol)
    (w2sys, w2sols) = witness_set_of_hypersurface(2, hyp2, precision)
    print('the witness sets for', hyp2)
    for pol in w2sys:
        print(pol)
    for sol in w2sols:
        print(sol)
    (sys, sols) = diagonal_solver\
        (2, 1, w1sys, w1sols, 1, w2sys, w2sols, 0, precision)
    print('the end system :')
    for pol in sys:
        print(pol)
    print('the solutions of the diagonal solver :')
    for sol in sols:
        print(sol)

def test():
    """
    Runs a test on algebraic sets.
    """
    from phcpy.phcpy2c3 import py2c_set_seed
    py2c_set_seed(234798272)
    test_diaghom('d')

if __name__ == "__main__":
    test()



"""
The module tropisms exports function to manage numerically computed
tropisms in double, double double, or quad double precision.
"""

def standard_initialize(nbt, dim, wnd, dir, err):
    r"""
    Initializes the direction vectors computed in double precision,
    along with estimates for their winding numbers and errors.
    On entry are the following five parameters:

    *nbt*: the number of direction vectors;

    *dim*: the number of coordinates in each vector;

    *wnd*: a list of integer values for the winding numbers, as many as *nbt*;

    *dir*: a list of lists of doubles with the coordinates of the directions,
    each inner list has *dim* doubles and *nbt* vectors are given;

    *err*: a list of *nbt* doubles.
    """
    from phcpy.phcpy2c2 import py2c_numbtrop_standard_initialize as store
    flat = []
    for vec in dir:
        flat = flat + vec
    data = wnd + flat + err
    store(nbt, dim, str(data))

def dobldobl_initialize(nbt, dim, wnd, dir, err):
    r"""
    Initializes the direction vectors computed in double double precision,
    along with estimates for their winding numbers and errors.
    On entry are the following five parameters:

    *nbt*: the number of direction vectors;

    *dim*: the number of coordinates in each vector;

    *wnd*: a list of integer values for the winding numbers, as many as *nbt*;

    *dir*: a list of lists of double doubles with the coordinates of the 
    directions, each inner list has *dim* double doubles and *nbt* vectors;

    *err*: a list of nbt double doubles.
    """
    from phcpy.phcpy2c2 import py2c_numbtrop_dobldobl_initialize as store
    flat = []
    for vec in dir:
        flat = flat + vec
    data = wnd + flat + err
    store(nbt, dim, str(data))

def quaddobl_initialize(nbt, dim, wnd, dir, err):
    r"""
    Initializes the direction vectors computed in quad double precision,
    along with estimates for their winding numbers and errors.
    On entry are the following five parameters:

    *nbt*: the number of direction vectors;

    *dim*: the number of coordinates in each vector;

    *wnd*: a list of integer values for the winding numbers, as many as *nbt*;

    *dir*: a list of lists of quad doubles with the coordinates of the 
    directions, each inner list has *dim* quad doubles and *nbt* vectors;

    *err*: a list of nbt quad doubles.
    """
    from phcpy.phcpy2c2 import py2c_numbtrop_quaddobl_initialize as store
    flat = []
    for vec in dir:
        flat = flat + vec
    data = wnd + flat + err
    store(nbt, dim, str(data))

def standard_size():
    """
    Returns the number of tropisms stored in standard double precision.
    """
    from phcpy.phcpy2c2 import py2c_numbtrop_standard_size as get_size
    return get_size()

def dobldobl_size():
    """
    Returns the number of tropisms stored in double double precision.
    """
    from phcpy.phcpy2c2 import py2c_numbtrop_dobldobl_size as get_size
    return get_size()

def quaddobl_size():
    """
    Returns the number of tropisms stored in quad double precision.
    """
    from phcpy.phcpy2c2 import py2c_numbtrop_quaddobl_size as get_size
    return get_size()

def standard_dimension():
    """
    Returns the dimension of the tropisms stored in standard double precision.
    """
    from phcpy.phcpy2c2 import py2c_numbtrop_standard_dimension as get_dim
    return get_dim()

def dobldobl_dimension():
    """
    Returns the dimension of the tropisms stored in double double precision.
    """
    from phcpy.phcpy2c2 import py2c_numbtrop_dobldobl_dimension as get_dim
    return get_dim()

def quaddobl_dimension():
    """
    Returns the dimension of the tropisms stored in quad double precision.
    """
    from phcpy.phcpy2c2 import py2c_numbtrop_quaddobl_dimension as get_dim
    return get_dim()

def standard_retrieve(nbt, dim):
    r"""
    Given on input the number of tropisms in *nbt* and the dimension in *dim*,
    returns a tuple of three lists: the winding numbers, coordinates of
    the direction vectors, and the errrors; in standard double precision.
    """
    from ast import literal_eval
    from phcpy.phcpy2c2 import py2c_numbtrop_standard_retrieve as load
    (fail, strdata) = load(nbt, dim)
    data = literal_eval(strdata)
    wnd = [int(data[k]) for k in range(nbt)]
    dirs = []
    for i in range(nbt):
        dirs.append([data[nbt+i*dim+j] for j in range(dim)])
    err = [data[nbt*(dim+1)+k] for k in range(nbt)]
    return (wnd, dirs, err)

def dobldobl_retrieve(nbt, dim):
    r"""
    Given on input the number of tropisms in *nbt* and the dimension in *dim*,
    returns a tuple of three lists: the winding numbers, coordinates of
    the direction vectors, and the errrors; in double double precision.
    """
    from ast import literal_eval
    from phcpy.phcpy2c2 import py2c_numbtrop_dobldobl_retrieve as load
    (fail, strdata) = load(nbt, dim)
    data = literal_eval(strdata)
    wnd = [int(data[k]) for k in range(nbt)]
    dirs = []
    for i in range(nbt):
        dirs.append([data[nbt+i*2*dim+j] for j in range(2*dim)])
    err = [data[nbt*(2*dim+1)+k] for k in range(2*nbt)]
    return (wnd, dirs, err)

def quaddobl_retrieve(nbt, dim):
    r"""
    Given on input the number of tropisms in *nbt* and the dimension in *dim*,
    returns a tuple of three lists: the winding numbers, coordinates of
    the direction vectors, and the errrors; in quad double precision.
    """
    from ast import literal_eval
    from phcpy.phcpy2c2 import py2c_numbtrop_quaddobl_retrieve as load
    (fail, strdata) = load(nbt, dim)
    data = literal_eval(strdata)
    wnd = [int(data[k]) for k in range(nbt)]
    dirs = []
    for i in range(nbt):
        dirs.append([data[nbt+i*4*dim+j] for j in range(4*dim)])
    err = [data[nbt*(4*dim+1)+k] for k in range(4*nbt)]
    return (wnd, dirs, err)

def retrieve_standard_tropism(dim, idx):
    r"""
    Returns the winding number, coordinates of the direction, and its error,
    stored in double precision, of dimension *dim*, and index *idx*.
    The index *idx* must be in the range 1..standard_size().
    Observe that the index counter starts at one and not at zero.
    """
    from ast import literal_eval
    from phcpy.phcpy2c2 \
    import py2c_numbtrop_standard_retrieve_tropism as load
    (fail, wnd, strdata) = load(dim, idx)
    data = literal_eval(strdata)
    dir = [data[k] for k in range(dim)]
    err = data[dim]
    return (wnd, dir, err)

def retrieve_dobldobl_tropism(dim, idx):
    r"""
    Returns the winding number, coordinates of the direction, and its error,
    stored in double precision, of dimension *dim*, and index *idx*.
    The index *idx* must be in the range 1..dobldobl_size().
    Observe that the index counter starts at one and not at zero.
    """
    from ast import literal_eval
    from phcpy.phcpy2c2 \
    import py2c_numbtrop_dobldobl_retrieve_tropism as load
    (fail, wnd, strdata) = load(dim, idx)
    data = literal_eval(strdata)
    dir = [data[k] for k in range(2*dim)]
    err = [data[2*dim], data[2*dim+1]]
    return (wnd, dir, err)

def retrieve_quaddobl_tropism(dim, idx):
    r"""
    Returns the winding number, coordinates of the direction, and its error,
    stored in double precision, of dimension *dim*, and index *idx*.
    The index *idx* must be in the range 1..quaddobl_size().
    Observe that the index counter starts at one and not at zero.
    """
    from ast import literal_eval
    from phcpy.phcpy2c2 \
    import py2c_numbtrop_quaddobl_retrieve_tropism as load
    (fail, wnd, strdata) = load(dim, idx)
    data = literal_eval(strdata)
    dir = [data[k] for k in range(4*dim)]
    err = [data[4*dim], data[4*dim+1], data[4*dim+2], data[4*dim+3]]
    return (wnd, dir, err)

def store_standard_tropism(dim, idx, wnd, dir, err):
    r"""
    Stores the tropism, given in standard double precision,
    with *dim* doubles as coordinates in the list *dir*, the error in *err*,
    and the winding number *wnd*, at position *idx*.
    The index *idx* must be in the range 1..standard_size().
    """
    from phcpy.phcpy2c2 import py2c_numbtrop_store_standard_tropism as store
    data = [x for x in dir]
    data.append(err)
    strdata = str(data)
    store(dim, idx, wnd, strdata)

def store_dobldobl_tropism(dim, idx, wnd, dir, err):
    r"""
    Stores the tropism, given in standard double precision,
    with *dim* doubles as coordinates in the list *dir*, the error in *err*,
    and the winding number *wnd*, at position *idx*.
    The index *idx* must be in the range 1..dobldobl_size().
    """
    from phcpy.phcpy2c2 import py2c_numbtrop_store_dobldobl_tropism as store
    data = [x for x in dir]
    data.append(err)
    strdata = str(data)
    store(dim, idx, wnd, strdata)

def store_quaddobl_tropism(dim, idx, wnd, dir, err):
    r"""
    Stores the tropism, given in standard double precision,
    with *dim* doubles as coordinates in the list *dir*, the error in *err*,
    and the winding number *wnd*, at position *idx*.
    The index *idx* must be in the range 1..quaddobl_size().
    """
    from phcpy.phcpy2c2 import py2c_numbtrop_store_quaddobl_tropism as store
    data = [x for x in dir]
    data.append(err)
    strdata = str(data)
    store(dim, idx, wnd, strdata)

def standard_clear():
    """
    Clears the tropisms stored in standard double precision.
    """
    from phcpy.phcpy2c2 import py2c_numbtrop_standard_clear
    py2c_numbtrop_standard_clear()

def dobldobl_clear():
    """
    Clears the tropisms stored in double double precision.
    """
    from phcpy.phcpy2c2 import py2c_numbtrop_dobldobl_clear
    py2c_numbtrop_dobldobl_clear()

def quaddobl_clear():
    """
    Clears the tropisms stored in quad double precision.
    """
    from phcpy.phcpy2c2 import py2c_numbtrop_quaddobl_clear
    py2c_numbtrop_quaddobl_clear()

def test_standard_store_load():
    """
    Tests the storing and loading of numerically computed tropisms,
    in standard double precision.
    """
    print 'testing storing and loading numerical tropisms...'
    nbr = int(input('Give the number of tropisms : '))
    dim = int(input('Give the dimension : '))
    from random import randint
    wnd = [randint(10, 99) for _ in range(nbr)]
    from random import uniform as u
    dirs = []
    for _ in range(nbr):
        dirs.append([u(-1, +1) for _ in range(dim)])
    errs = [u(0,1)/1000.0 for _ in range(nbr)]
    print 'random winding numbers :', wnd
    for k in range(len(dirs)):
        print 'direction', k+1, ':', dirs[k]
    print 'random errors :', errs
    standard_initialize(nbr, dim, wnd, dirs, errs)
    (retwnd, retdirs, reterrs) = standard_retrieve(nbr, dim)
    retsize = standard_size()
    retdim = standard_dimension()
    print 'retrieved number of tropisms :', retsize
    print 'retrieved dimension of the tropisms :', retdim
    print 'retrieved winding numbers :', retwnd
    print 'retrieved directions :'
    for k in range(len(retdirs)):
        print 'direction', k+1, ':', retdirs[k]
    print 'retrieved errors :', reterrs
    while True:
        idx = int(input('Give an index (0 to exit): '))
        if idx < 1:
            break
        (idxwnd, idxdir, idxerr) = retrieve_standard_tropism(dim, idx)
        print '-> winding number for tropism', idx, ':', idxwnd
        print '-> tropism', idx, 'has coordinates :', idxdir
        print '-> the error :', idxerr
        ranwnd = randint(10, 99)
        randir = [u(-1, 1) for _ in range(dim)]
        ranerr = u(0, 1)/1000.0
        print 'store tropism %d with winding number %d' % (idx,ranwnd)
        print 'with new coordinates :', randir
        print 'and error', ranerr
        store_standard_tropism(dim, idx, ranwnd, randir, ranerr)
    standard_clear()

def test_dobldobl_store_load():
    """
    Tests the storing and loading of numerically computed tropisms,
    in double double precision.
    """
    print 'testing storing and loading numerical tropisms...'
    nbr = int(input('Give the number of tropisms : '))
    dim = int(input('Give the dimension : '))
    from random import randint
    wnd = [randint(10, 99) for _ in range(nbr)]
    from random import uniform as u
    dirs = []
    for _ in range(nbr):
        dirs.append([u(-1, +1) for _ in range(2*dim)])
    errs = [u(0,1)/1000.0 for _ in range(2*nbr)]
    print 'random winding numbers :', wnd
    for k in range(len(dirs)):
        print 'direction', k+1, ':', dirs[k]
    print 'random errors :', errs
    dobldobl_initialize(nbr, dim, wnd, dirs, errs)
    (retwnd, retdirs, reterrs) = dobldobl_retrieve(nbr, dim)
    retsize = dobldobl_size()
    retdim = dobldobl_dimension()
    print 'retrieved number of tropisms :', retsize
    print 'retrieved dimension of the tropisms :', retdim
    print 'retrieved winding numbers :', retwnd
    print 'retrieved directions :'
    for k in range(len(retdirs)):
        print 'direction', k+1, ':', retdirs[k]
    print 'retrieved errors :', reterrs
    while True:
        idx = int(input('Give an index (0 to exit): '))
        if idx < 1:
            break
        (idxwnd, idxdir, idxerr) = retrieve_dobldobl_tropism(dim, idx)
        print '-> winding number for tropism', idx, ':', idxwnd
        print '-> tropism', idx, 'has coordinates :', idxdir
        print '-> the error :', idxerr
        ranwnd = randint(10, 99)
        randir = [u(-1, 1) for _ in range(2*dim)]
        ranerr = [u(0, 1)/1000.0, u(0, 1)/1000.0]
        print 'store tropism %d with winding number %d' % (idx,ranwnd)
        print 'with new coordinates :', randir
        print 'and error', ranerr
        store_dobldobl_tropism(dim, idx, ranwnd, randir, ranerr)
    dobldobl_clear()

def test_quaddobl_store_load():
    """
    Tests the storing and loading of numerically computed tropisms,
    in quad double precision.
    """
    print 'testing storing and loading numerical tropisms...'
    nbr = int(input('Give the number of tropisms : '))
    dim = int(input('Give the dimension : '))
    from random import randint
    wnd = [randint(10, 99) for _ in range(nbr)]
    from random import uniform as u
    dirs = []
    for _ in range(nbr):
        dirs.append([u(-1, +1) for _ in range(4*dim)])
    errs = [u(0,1)/1000.0 for _ in range(4*nbr)]
    print 'random winding numbers :', wnd
    for k in range(len(dirs)):
        print 'direction', k+1, ':', dirs[k]
    print 'random errors :', errs
    quaddobl_initialize(nbr, dim, wnd, dirs, errs)
    (retwnd, retdirs, reterrs) = quaddobl_retrieve(nbr, dim)
    retsize = quaddobl_size()
    retdim = quaddobl_dimension()
    print 'retrieved number of tropisms :', retsize
    print 'retrieved dimension of the tropisms :', retdim
    print 'retrieved winding numbers :', retwnd
    print 'retrieved directions :'
    for k in range(len(retdirs)):
        print 'direction', k+1, ':', retdirs[k]
    print 'retrieved errors :', reterrs
    while True:
        idx = int(input('Give an index (0 to exit): '))
        if idx < 1:
            break
        (idxwnd, idxdir, idxerr) = retrieve_quaddobl_tropism(dim, idx)
        print '-> winding number for tropism', idx, ':', idxwnd
        print '-> tropism', idx, 'has coordinates :', idxdir
        print '-> the error :', idxerr
        ranwnd = randint(10, 99)
        randir = [u(-1, 1) for _ in range(4*dim)]
        ranerr = [u(0, 1)/1000.0, u(0, 1)/1000.0]
        print 'store tropism %d with winding number %d' % (idx,ranwnd)
        print 'with new coordinates :', randir
        print 'and error', ranerr
        store_quaddobl_tropism(dim, idx, ranwnd, randir, ranerr)
    quaddobl_clear()

def management_test():
    """
    Tests the management of numerically computed tropisms.
    """
    print 'testing management of numerical tropisms...'
    prc = input('give the precision (d, dd, or qd) : ')
    if prc == 'd':
        test_standard_store_load()
    elif prc == 'dd':
        test_dobldobl_store_load()
    else:
        test_quaddobl_store_load()

def test(precision='d'):
    """
    Tests the numerical computation of a tropism.
    """
    pols = ['x + x*y + y^3;', 'x;']
    from phcpy.solver import solve
    start = ['x + (-5.89219623474258E-02 - 9.98262591883082E-01*i)*y^2' \
         + ' + ( 5.15275165825429E-01 + 8.57024797472965E-01*i)*x*y;', \
        'x+(-9.56176913648087E-01 - 2.92789531586119E-01*i);']
    startsols = solve(start, verbose=False)
    print 'computed', len(startsols), 'start solutions'
    from phcpy.tuning import order_endgame_extrapolator_set as set
    set(4)
    if precision == 'd':
        from phcpy.trackers import standard_double_track as track
    elif precision == 'dd':
        from phcpy.trackers import double_double_track as track
    elif precision == 'qd':
        from phcpy.trackers import quad_double_track as track
    else:
        print 'wrong level of precision'
        return
    sols = track(pols, start, startsols)
    print 'the solutions at the end :'
    for sol in sols:
        print sol
    if precision == 'd':
        print 'size of the tropisms container :', standard_size()
        (wnd, dirs, errs) = standard_retrieve(len(sols), len(pols))
    elif precision == 'dd':
        print 'size of the tropisms container :', dobldobl_size()
        (wnd, dirs, errs) = dobldobl_retrieve(len(sols), len(pols))
    else:
        print 'size of the tropisms container :', quaddobl_size()
        (wnd, dirs, errs) = quaddobl_retrieve(len(sols), len(pols))
    print 'the winding numbers :', wnd
    print 'the directions :'
    for dir in dirs:
        print dir
    print 'the errors :', errs
    # check which tuning parameters were used
    # from phcpy.tuning import tune_track_parameters as tune
    # tune(interactive=True)

if __name__ == "__main__":
    test('d')



"""
Excerpt from the user manual of phcpy.
"""
import matplotlib.pyplot as plt
from phcpy.dimension import set_seed, get_seed
from phcpy.solutions import strsol2dict
from phcpy.starters import total_degree_start_system
from phcpy.trackers import initialize_double_tracker
from phcpy.trackers import initialize_double_solution
from phcpy.trackers import next_double_solution

set_seed(12871)
print('the seed :', get_seed())
p = ['x^2 + y - 3;', 'x + 0.125*y^2 - 1.5;']
print('constructing a total degree start system ...')
q, qsols = total_degree_start_system(p)
print('number of start solutions :', len(qsols))
initialize_double_tracker(p, q, False)
plt.ion()
fig = plt.figure()
for k in range(len(qsols)):
    if(k == 0):
        axs = fig.add_subplot(221)
    elif(k == 1):
        axs = fig.add_subplot(222)
    elif(k == 2):
        axs = fig.add_subplot(223)
    elif(k == 3):
        axs = fig.add_subplot(224)
    startsol = qsols[k]
    initialize_double_solution(len(p),startsol)
    dictsol = strsol2dict(startsol)
    xpoints =  [dictsol['x']]
    ypoints =  [dictsol['y']]
    for k in range(300):
        ns = next_double_solution()
        dictsol = strsol2dict(ns)
        xpoints.append(dictsol['x'])
        ypoints.append(dictsol['y'])
        tval = dictsol['t'].real
        if(tval == 1.0):
            break
    print(ns)
    xre = [point.real for point in xpoints]
    yre = [point.real for point in ypoints]
    axs.set_xlim(min(xre)-0.3, max(xre)+0.3)
    axs.set_ylim(min(yre)-0.3, max(yre)+0.3)
    dots, = axs.plot(xre,yre,'r-')
    dots, = axs.plot(xre,yre,'ro')
    fig.canvas.draw()
fig.canvas.draw()
ans = input('hit return to exit')



"""
PHCpy --- a package for Polynomial Homotopy Continuation
========================================================

PHCpy is a collection of Python modules to compute solutions
of polynomial systems using PHCpack.

A homotopy defines the deformation of a start system
(system with known solutions) into the target system
(system that has to be solved).
Continuation or path tracking methods apply numerical
predictor-corrector techniques to track the solution paths
defined by the homotopy, starting at the known solutions of the
start system and ending at the solutions of the target system.

Available modules
-----------------
solver
   exports the blackbox solver of PHCpack, a mixed volume calculator,
   a path tracker, functions to construct start systems, and deflation
   to recondition isolated singular solutions.
solutions
   solutions of phcpy.solve are lists of PHCpack solution strings
   and this module exports operations to convert the solution
   strings into Python dictionaries, e.g. for evaluation.
polynomials
   the object oriented interface starts with the class Polynomials,
   which represents a system of polynomials and provides a solve method.
interface
   data transfer from string representations of polynomials and solutions
   as the interface between Python and the C interface of PHCpack.
trackers
   offers functions to track solution paths defined by a homotopy between
   a given start system with known solutions and a target system.
sweepers
   homotopies to sweep a real parameter range in natural parameter homotopies.
tuning
   parameters and numerical tolerances of the trackers and sweepers
   are tuned by the functions in the tuning module.
sets
   offers tools to work with positive dimensional solution sets.
cascades
   homotopies to compute candidate generic points on all components.
factor
   separates generic points in a witness set according to the
   irreducible factors in a solution set.
diagonal
   given two witness sets, diagonal homotopies compute the intersection
   of the given sets.
examples
   defines some interesting examples taken from the research literature,
   the test() solves all systems, performing a regression test.
families
   polynomial system often occur in families and are defined for any
   number of equations and variables, e.g.: the cyclic n-roots system.
schubert
   exports the hypersurface and quantum Pieri homotopies to compute
   isolated solutions to problems in enumerative geometry.
polytopes
   functions to work with Newton polytopes, to compute mixed volumes
   of Newton polytopes, given by tuples of support sets.
tropisms
   tropisms are the leading exponents of the power series solutions
   and can be computed by polyhedral end games.
maps
   module to work with monomial maps, defined as solution of systems
   that have exactly two monomials in every equation (binomial systems).
series
   Newton's method on truncated power series solutions.
curves
   approximate algebraic curves with rational expressions: Pade approximants
dashboard
   prototype of a graphical user interface with Tkinter.
server
   defines a simple client/server interaction to solve random trinomials.

Calling the blackbox solver
---------------------------

Polynomials and solutions are represented as strings.
Below is an illustration of a session with the blackbox solver
on a system of two random trinomials, polynomials with three
monomials with random coefficients.

   >>> from phcpy.solver import random_trinomials
   >>> f = random_trinomials()
   >>> print f[0]
   (0.583339727743+0.81222826966115*i)*x^0*y^0\
+(-0.730410130891-0.68300881450520*i)*x^5*y^5\
+(0.547878834338+0.83655769847920*i)*x^5*y^0;
   >>> print f[1]
   (0.830635910813+0.55681593338247*i)*x^0*y^4\
+(0.456430547798-0.88975904324518*i)*x^1*y^4\
+(0.034113254002-0.99941797357332*i)*x^2*y^1;
   >>> from phcpy.solver import solve
   >>> s = solve(f,silent=True)
   >>> len(s)
   30
   >>> print s[2]
   t :  1.00000000000000E+00   0.00000000000000E+00
   m : 1
   the solution for t :
    x : -9.99963006604849E-01   8.60147787997449E-03
    y :  0.00000000000000E+00   0.00000000000000E+00
   == err :  4.325E-17 = rco :  2.020E-01 = res :  1.665E-16 =
   >>>

The solve command returned a list of 30 strings in s,
each string represents a solution that makes the polynomials in f vanish.
The module solutions offers function to evaluate the solutions
in the polynomials given as strings.
"""
def cite():
    """
    Displays the citation information for phcpy.
    """
    print """
    To cite phcpy in publications use:

    Jan Verschelde: Modernizing PHCpack through phcpy.
    In Proceedings of the 6th European Conference on Python in Science
    (EuroSciPy 2013), edited by Pierre de Buyl and Nelle Varoquaux,
    pages 71-76, 2014, available at http://arxiv.org/abs/1310.0056.
    """

try:
    from phcpy2c2 import py2c_PHCpack_version_string
    print py2c_PHCpack_version_string() + ' works!'
    from phcpy import solver, solutions, interface, trackers, sweepers, tuning
    from phcpy import sets, cascades, factor, diagonal, schubert
    from phcpy import polytopes, polynomials, tropisms, maps, series, curves
    from phcpy import examples, families
    # for Sage, uncomment the following two lines
    # from cysignals import init_cysignals
    # init_cysignals()
except:
    print 'Is the phcpy2c2.so not suited for this platform?'

# The version number is defined as a data attribute.
__version__ = '1.1.2'



"""
The script tests the Littlewood-Richardson homotopies on some
relatively big problem in G(4,8).  The ten problems were formulated
by Frank Sottile at an AIM squares meeting in the Fall of 2015.
(0) [3 6 7 8]^7*[4 6 7 8]^2 = 231
(1) [2 5 6 8]*[3 6 7 8]*[4 5 7 8]*[4 6 7 8]^7 = 294
(2) [2 4 7 8]*[3 6 7 8]^2*[4 6 7 8]^7 = 356
(3) [2 5 7 8]^2*[4 6 7 8]^8 = 398
(4) [3 5 7 8]*[4 5 7 8]^2*[3 6 7 8]^2*[4 6 7 8]^5 = 526
(5) [4 5 7 8]^3*[3 6 7 8]^2*[4 6 7 8]^6 = 734
(6) [3 5 7 8]*[3 6 7 8]^3*[4 6 7 8]^7 = 1104
(7) [3 6 7 8]^4*[4 6 7 8]^8 = 1600
(8) [3 5 7 8]*[3 6 7 8]^2*[4 6 7 8]^9 = 2166
(9) [3 6 7 8]^3*[4 6 7 8]^10 = 3102
"""

def ten_problems():
    """
    Returns a list of ten Schubert problems in G(4, 8) as a list
    of tuples.  The first element in each tuple contains the brackets
    and the second element the corresponding root count.
    """
    result = []
    b0 = [[3, 6, 7, 8] for _ in range(7)] \
       + [[4, 6, 7, 8] for _ in range(2)]
    r0 = 231
    result.append((b0, r0))
    b1 = [[2, 5, 6, 8], [3, 6, 7, 8], [4, 5, 7, 8]] \
       + [[4, 6, 7, 8] for _ in range(7)]
    r1 = 294
    result.append((b1, r1))
    b2 = [[2, 4, 7, 8]] + [[3, 6, 7, 8] for _ in range(2)] \
       + [[4, 6, 7, 8] for _ in range(7)]
    r2 = 356
    result.append((b2, r2))
    b3 = [[2, 5, 7, 8] for _ in range(2)] \
       + [[4, 6, 7, 8] for _ in range(8)]
    r3 = 398
    result.append((b3, r3))
    b4 = [[3, 5, 7, 8]] \
       + [[4, 5, 7, 8] for _ in range(2)] \
       + [[3, 6, 7, 8] for _ in range(2)] \
       + [[4, 6, 7, 8] for _ in range(5)]
    r4 = 526
    result.append((b4, r4))
    b5 = [[4, 5, 7, 8] for _ in range(3)] \
       + [[3, 6, 7, 8] for _ in range(2)] \
       + [[4, 6, 7, 8] for _ in range(6)]
    r5 = 734
    result.append((b5, r5))
    b6 = [[3, 5, 7, 8]] \
       + [[3, 6, 7, 8] for _ in range(3)] \
       + [[4, 6, 7, 8] for _ in range(7)]
    r6 = 1104
    result.append((b6, r6))
    b7 = [[3, 6, 7, 8] for _ in range(4)] \
       + [[4, 6, 7, 8] for _ in range(8)]
    r7 = 1600
    result.append((b7, r7))
    b8 = [[3, 5, 7, 8]] \
       + [[3, 6, 7, 8] for _ in range(2)] \
       + [[4, 6, 7, 8] for _ in range(9)]
    r8 = 2166
    result.append((b8, r8))
    b9 = [[3, 6, 7, 8] for _ in range(3)] \
       + [[4, 6, 7, 8] for _ in range(10)]
    r9 = 3102
    result.append((b9, r9))
    return result

def count_roots(problems):
    """
    Runs the root count on the ten problems.
    Checks whether the computed root corresponds with the exact one.
    An exception will be raised if an error occurs in the check.
    """
    from phcpy.schubert import resolve_schubert_conditions as root_count
    (k, n) = (4, 8) # in G(4,8)
    for (brackets, count) in problems:
        c = root_count(n, k, brackets)
        print(brackets, '=', c)
        assert(count == c)

def solve(problems):
    """
    Runs the Littlewood-Richardson homotopies on the problems.
    """
    from phcpy.schubert import littlewood_richardson_homotopies as lrhom
    (k, n) = (4, 8) # in G(4,8)
    for (brackets, count) in problems:
        print(brackets, '=', count)
        (rc, flags, pols, sols) = lrhom(n, k, brackets)
        assert(len(sols) == count)

def main():
    """
    Generates ten problems and counts the roots.
    """
    prbs = ten_problems()
    count_roots(prbs)
    solve(prbs)

main()



"""
The script demonstrates the speedup of multithreading.
To measure speedup, we compare wall clock times.
In both runs, the seed is fixed to work 
with the same random constants in the homotopies.
"""
from datetime import datetime
from phcpy.dimension import get_core_count, set_seed
from phcpy.polynomials import set_double_system
from phcpy.families import cyclic
from phcpy.volumes import mixed_volume
from phcpy.solver import solve

nbcores = get_core_count()
print('number of available cores :', nbcores)
print('the cyclic 7-roots problem :')
c7 = cyclic(7)
for pol in c7:
    print(pol)
print('the mixed volume :', mixed_volume(c7))
print('solving on one core ...')
set_seed(2024)
timestart = datetime.now()
s = solve(c7)
timestop = datetime.now()
elapsed_onecore = timestop - timestart
print('computed', len(s), 'solutions in time', elapsed_onecore)
print('solving on', nbcores, 'cores ...')
set_seed(2024)
timestart = datetime.now()
s = solve(c7, tasks=nbcores)
timestop = datetime.now()
elapsed_manycores = timestop - timestart
print('computed', len(s), 'solutions in time', elapsed_manycores)
speedup = elapsed_onecore/elapsed_manycores
print('the speedup : {:2.2f}'.format(speedup))
